@@ -1,11 +1,14 @@
 /*-----------------------------------------------------------------------*/
 /* formats.c --- input file support for xcircuit			 */
-/* Copyright (c) 1998  Tim Edwards, Johns Hopkins University        	 */
+/* Copyright (c) 2001  Tim Edwards, Johns Hopkins University        	 */
 /*-----------------------------------------------------------------------*/
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
+#if !defined(__DARWIN__)
 #include <malloc.h>
+#endif
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
 
@@ -16,6 +19,11 @@
 #include "colordefs.h"
 #include "xcircuit.h"
 
+/*----------------------------------------------------------------------*/
+/* Function prototype declarations                                      */
+/*----------------------------------------------------------------------*/
+#include "prototypes.h"
+
 /*------------------------------------------------------------------------*/
 /* Useful (local) defines						  */
 /*------------------------------------------------------------------------*/
@@ -30,42 +38,31 @@
 /*------------------------------------------------------------------------*/
 
 extern char _STR[150];
+extern Globaldata xobjs;
 extern Clientdata areastruct;
 extern Display *dpy;
 extern int *appcolors;
 extern short pushes;
 
-/*------------------------------------------------------------------------*/
-/* Declarations of functions defined externally to functions.c            */
-/*------------------------------------------------------------------------*/
-
-extern void zoomview(), reset(), topreset(), initmem();
-extern Boolean nextfilename();
-
-/*-------------------------*/
+/*----------------------------------------------*/
 #ifdef LGF
 
-/*-------------------------*/
-/* Load an analog LGF file */
-/*-------------------------*/
-
 Boolean lgfisloaded = False;
 
-void loadlgf(nullval)
-  caddr_t nullval;
+/*----------------------------------------------*/
+/* loadlgf: Load an analog LGF file 		*/
+/*	mode = 1: import into current page;	*/
+/*	mode = 0 clear current page and load	*/
+/*----------------------------------------------*/
+
+void loadlgf(int mode)
 {
    FILE *ps;
-   char inname[150], temp[500], keyword[30], percentc, *pdchar;
+   char inname[150], temp[500], *pdchar;
    char **signals;
    short *signets;
-   char teststr[20], teststr2[20];
-   short offx, offy;
-   objectptr	*libobj, *dotobj;
+   objectptr *libobj;
    genericptr *iolabel;
-   float tmpfl;
-   short tmpfont = 0;
-   short founddot = 0;
-   float tmpscale;
    int i, sigs;
 
    sscanf(_STR, "%s", inname);
@@ -85,33 +82,20 @@ void loadlgf(nullval)
       }
    }
 
-   sprintf(_STR, "Loaded file: %s", inname);
-   Wprintf(_STR);
-
-   /* for PostScript file, remove ".lgf" or ".lfo" and replace with ".ps" */
+   /* for PostScript file, remove ".lgf" or ".lfo" (to be replaced with ".ps") */
 
    if ((pdchar = strstr(inname, ".l")) != NULL) *pdchar = '\0';
 
-   areastruct.filename[areastruct.page] = (char *) realloc (
-      areastruct.filename[areastruct.page], (strlen(inname) + 1) * sizeof(char));
-   strcpy(areastruct.filename[areastruct.page], inname);
-
-   /* If the filename has a path component, use only the root */
-
-   if ((pdchar = strrchr(inname, '/')) != NULL)
-      sprintf(objectdata->name, "%s", pdchar + 1);
-   else
-      sprintf(objectdata->name, "%s", inname);
-
-   renamepage(areastruct.page);
-   printname(objectdata);
+   sprintf(_STR, "Loaded file: %s", inname);
+   Wprintf(_STR);
 
    /* Make sure that LGF object library has been loaded by loading it now. */
 
    if (!lgfisloaded) {
+      int ilib;
       strcpy(_STR, LGF_LIB);
-      loadlibrary(LIBRARY);
-      composelib(LIBRARY);
+      ilib = createlibrary();
+      loadlibrary(ilib);
       lgfisloaded = True;
    }
    
@@ -142,16 +126,28 @@ void loadlgf(nullval)
       return;
    }
 
-   /* Done with header */
+   /* Done with header. . . okay to clear current page now unless importing */
 
-   /* Find the object "dot" in the builtin library, or else use an arc */
+   if (mode == 0) {
+      reset(objectdata, NORMAL);
+      topreset(); 
+   }
 
-   for (dotobj = areastruct.library; dotobj != areastruct.library +
-	   areastruct.builtins; dotobj++)
-      if (!strcmp((*dotobj)->name, "dot")) {
-	 founddot = 1;
-         break;
-      }
+   /* Set up filename and object (page) name */
+
+   xobjs.pagelist[areastruct.page]->filename = (char *) realloc (
+      xobjs.pagelist[areastruct.page]->filename, (strlen(inname) + 1) * sizeof(char));
+   strcpy(xobjs.pagelist[areastruct.page]->filename, inname);
+
+   /* If the filename has a path component, use only the root */
+
+   if ((pdchar = strrchr(inname, '/')) != NULL)
+      sprintf(objectdata->name, "%s", pdchar + 1);
+   else
+      sprintf(objectdata->name, "%s", inname);
+
+   renamepage(areastruct.page);
+   printname(objectdata);
 
    /* Read objects */
 
@@ -183,10 +179,7 @@ void loadlgf(nullval)
 	    }
 	    break;
 
-	 case 's': {	/* signal names --- save for future reference */
-
-	    labelptr *newlabel;
-	    char *tstrp;
+	 case 's': 	/* signal names --- save for future reference */
 
 	    sscanf(++lineptr, "%d", &sigs);
 	    signals = (char **) malloc(sigs * sizeof(char *));
@@ -204,7 +197,7 @@ void loadlgf(nullval)
 
 	       signals[i] = (char *)malloc((strlen(tmpstring) + 1) * sizeof(char));
 	       sprintf(signals[i], "%s", tmpstring);
-            }}
+            }
 	    break;
 	 
 	 case 'l': {	/* labels */
@@ -243,20 +236,22 @@ void loadlgf(nullval)
 	       while (isspace(*(++tstrp)));
 
 	       if (tstrp != NULL) {	/* could be a blank line */
+		  stringpart *strptr;
 
 		  NEW_LABEL(newlabel, objectdata);
 	          objectdata->parts++;
 
-	          (*newlabel)->position.x = xmat(pvalx);
-	          (*newlabel)->position.y = ymat(pvaly);
-	          (*newlabel)->rotation = 1;
-
+		  labeldefaults(*newlabel, False, xmat(pvalx), ymat(pvaly));
 	          (*newlabel)->justify = TOP | NOTBOTTOM;
-	          (*newlabel)->scale = 1.0;
-		  (*newlabel)->color = FOREGROUND;
-	          (*newlabel)->string = (uchar *) malloc((strlen(tstrp) + 3) *
-                      sizeof(uchar));
-	          sprintf((*newlabel)->string, "%c%c%s", TEXT_ESC, FONT_START, tstrp);
+		  (*newlabel)->color = DEFAULTCOLOR;
+		  (*newlabel)->string->data.font = 0;
+		  strptr = makesegment(&((*newlabel)->string), NULL);
+		  strptr->type = TEXT_STRING;
+		  strptr->data.string = (char *)malloc(1 + strlen(tstrp));
+		  strcpy(strptr->data.string, tstrp);
+#ifdef SCHEMA
+		  (*newlabel)->pin = NORMAL;
+#endif
 	       }
             }}
 	    break;
@@ -285,7 +280,7 @@ void loadlgf(nullval)
 	       (*newwire)->points = (XPoint *)malloc(2 * sizeof(XPoint));
 	       (*newwire)->width = 1.0;
 	       (*newwire)->style = UNCLOSED;
-	       (*newwire)->color = FOREGROUND;
+	       (*newwire)->color = DEFAULTCOLOR;
 	       tmppnts = (*newwire)->points;
 	       tmppnts->x = xmat(pvalx);
 	       tmppnts->y = ymat(pvaly);
@@ -295,10 +290,7 @@ void loadlgf(nullval)
 	    }}
 	    break;
 
-	case 'p': {	/* solder dot */
-	    arcptr *newarc;
-	    objinstptr *newdot;
-	 
+	case 'p': 	/* solder dot */
 	    sscanf(++lineptr, "%d", &dval);
 	    for (i = 0; i < dval; i++) {
 	       do {
@@ -311,33 +303,9 @@ void loadlgf(nullval)
 
 	       /* Allocate arc */
 
-	       if (founddot == 1) {
-		  NEW_OBJINST(newdot, objectdata);
-	          objectdata->parts++;
-	          sscanf(lineptr, "%hd %hd", &pvalx, &pvaly);
-	          (*newdot)->position.x = xmat(pvalx);
-	          (*newdot)->position.y = ymat(pvaly);
-		  (*newdot)->rotation = 1;
-		  (*newdot)->scale = 1.0;
-		  (*newdot)->thisobject = *dotobj;
-		  (*newdot)->color = FOREGROUND;
-	       }
-	       else {
-		  NEW_ARC(newarc, objectdata);
-	          objectdata->parts++;
-	          sscanf(lineptr, "%hd %hd", &pvalx, &pvaly);
-	          (*newarc)->position.x = xmat(pvalx);
-	          (*newarc)->position.y = ymat(pvaly);
-	          (*newarc)->angle1 = 0;
-	          (*newarc)->angle2 = 360;
-	          (*newarc)->width = 1.0;
-	          (*newarc)->radius = 6;
-	          (*newarc)->yaxis = 6;
-	          (*newarc)->style = FILLED | FILLSOLID | NOBORDER;
-		  (*newarc)->color = FOREGROUND;
-		  calcarc(*newarc);
-	       }
-	    }}
+	       sscanf(lineptr, "%hd %hd", &pvalx, &pvaly);
+	       drawdot(xmat(pvalx), ymat(pvaly));
+	    }
 	    break;
 
 	case 'b': {	/* boxes */
@@ -358,7 +326,7 @@ void loadlgf(nullval)
 	       objectdata->parts++;
 
 	       (*newpoly)->style = DASHED;
-	       (*newpoly)->color = FOREGROUND;
+	       (*newpoly)->color = DEFAULTCOLOR;
 	       (*newpoly)->width = 1.0;
 	       (*newpoly)->number = 4;
                (*newpoly)->points = (pointlist) malloc(4 * sizeof(XPoint));
@@ -394,15 +362,14 @@ void loadlgf(nullval)
       	          for (lineptr = temp; isspace(*lineptr) && *lineptr != '\0'; lineptr++);
 	       } while (*lineptr == '\0');
 
-	       /* tricky double loop through builtin and user libraries */
+	       /* double loop through user libraries */
 
-	       for (libobj = areastruct.library; libobj != areastruct.userlib + 
-	              areastruct.userobjs; libobj++) {
-	          if (libobj == areastruct.library + areastruct.builtins) {
-		     libobj = areastruct.userlib;
-		     if (areastruct.userobjs == 0) break;
+	       for (j = 0; j < xobjs.numlibs; j++) {
+		  for (k = 0; k < xobjs.userlibs[j].number; k++) {
+		     libobj = xobjs.userlibs[j].library + k;
+	             if (!strcmp(lineptr, (*libobj)->name)) break;
 		  }
-	          if (!strcmp(lineptr, (*libobj)->name)) break;
+	          if (k < xobjs.userlibs[j].number) break;
 	       }
 	       strcpy(tmpstring, lineptr);
 
@@ -414,7 +381,7 @@ void loadlgf(nullval)
 	       }
       	       for (lineptr = temp; isspace(*lineptr) && *lineptr != '\n'; lineptr++);
 
-	       if (libobj != areastruct.userlib + areastruct.userobjs) {
+	       if (j < xobjs.numlibs || k < xobjs.userlibs[xobjs.numlibs - 1].number) {
 
 		  NEW_OBJINST(newinst, objectdata);
       	          objectdata->parts++;
@@ -428,7 +395,8 @@ void loadlgf(nullval)
 		  (*newinst)->position.x = xmat(pvalx);
 		  (*newinst)->position.y = ymat(pvaly);
 		  (*newinst)->scale = 1.0;
-		  (*newinst)->color = FOREGROUND;
+		  (*newinst)->color = DEFAULTCOLOR;
+		  (*newinst)->params = NULL;
 
 		  if (pvalx2 & 0x01) pvalx2 ^= 0x02;
                   if (pvalx2 >= 4) (*newinst)->rotation = -((pvalx2 - 4) *
@@ -447,6 +415,7 @@ void loadlgf(nullval)
 
 		     for (k = 0; k < sigs; k++)
 		        if (signets[k] == nval) {
+			   stringpart *strptr;
 
 			   NEW_LABEL(newlabel, objectdata);
 			   objectdata->parts++;
@@ -454,22 +423,27 @@ void loadlgf(nullval)
 			   newinst = (objinstptr *)(objectdata->plist
 				+ objectdata->parts - 2);
 
-			   (*newlabel)->position.x = (*newinst)->position.x;
+			   labeldefaults(*newlabel, False, (*newinst)->position.x,
+				(*newinst)->position.y);
+			   (*newlabel)->color = DEFAULTCOLOR;
+#ifdef SCHEMA
+		  	   if (areastruct.schemon){
+			      (*newlabel)->pin = LOCAL;
+			      (*newlabel)->color = SNAPCOLOR;
+		  	   }
+#endif
 			   if (!strcmp(tmpstring, "TO"))
 			      (*newlabel)->position.x += ((flip) ? 48 : -48);
 			   else
 			      (*newlabel)->position.x += ((flip) ? 54 : -54);
-			   (*newlabel)->position.y = (*newinst)->position.y;
-			   (*newlabel)->rotation = 1;
 
 			   (*newlabel)->justify = NOTBOTTOM;
 			   if (flip) (*newlabel)->justify |= (RIGHT | NOTLEFT);
-			   (*newlabel)->scale = 1.0;
-			   (*newlabel)->color = FOREGROUND;
-			   (*newlabel)->string = (uchar *) malloc((strlen(signals[k]) + 2)
-				   * sizeof(uchar));
-			   sprintf((*newlabel)->string, "%c%c%s", TEXT_ESC, FONT_START,
-				   signals[k]);
+			   (*newlabel)->string->data.font = 0;
+			   strptr = makesegment(&((*newlabel)->string), NULL);
+			   strptr->type = TEXT_STRING;
+			   strptr->data.string = (char *)malloc(1 + strlen(signals[k]));
+			   strcpy(strptr->data.string, signals[k]);
 			   break;
 		        }
 	          }
@@ -548,7 +522,8 @@ void loadlgf(nullval)
 	 }
       }
 
-   zoomview(NULL, Number(1), NULL);
+   calcbbox(objectdata);
+   centerview(objectdata);
 
    for (i = 0; i < sigs; i++) free(signals[i]);
    free(signals);
@@ -568,8 +543,7 @@ void loadlgf(nullval)
 /* This is unfinished. . . needs a lot of thought and a lot of work */
 /*------------------------------------------------------------------*/
 
-void loadmat4(nullval)
-  caddr_t nullval;
+void loadmat4(caddr_t nullval)
 {
    char inname[150], *temp, *buffer, keyword[30], percentc, *pdchar;
    int bufsize = 256;
