@@ -1,6 +1,6 @@
 /*-------------------------------------------------------------------------*/
 /* functions.c 								   */
-/* Copyright (c) 2000  Tim Edwards, Johns Hopkins University        	   */
+/* Copyright (c) 2001  Tim Edwards, Johns Hopkins University        	   */
 /*-------------------------------------------------------------------------*/
 
 /*-------------------------------------------------------------------------*/
@@ -8,8 +8,11 @@
 /*-------------------------------------------------------------------------*/
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
+#if !defined(__DARWIN__)
 #include <malloc.h>
+#endif
 #include <math.h>
 
 #include <X11/Intrinsic.h>
@@ -22,6 +25,11 @@
 #include "colordefs.h"
 #include "xcircuit.h"
 
+/*----------------------------------------------------------------------*/
+/* Function prototype declarations                                      */
+/*----------------------------------------------------------------------*/
+#include "prototypes.h"
+
 /*-------------------------------------------------------------------------*/
 /* External Variable definitions					   */
 /*-------------------------------------------------------------------------*/
@@ -37,20 +45,6 @@ extern short textpos, textend;
 extern objectpair *pushlist;
 extern short pushes;
 
-/*------------------------------------------------------------------------*/
-/* Declarations of functions defined externally to functions.c		  */
-/*------------------------------------------------------------------------*/
-
-extern TextExtents ULength();
-extern void UTransformPoints();
-
-void UTransformbyCTM();
-void calcbboxvalues(objectptr, uchar, genericptr *);
-
-#ifdef SCHEMA
-void pinadjust (short, short *, short *, short);
-#endif
-
 /*-----------------------*/
 /* Arithmetic functions  */
 /*-----------------------*/
@@ -106,8 +100,7 @@ float *sine[RSTEPS + 1] =  {
 /* user space).								  */
 /*------------------------------------------------------------------------*/
 
-long sqwirelen(userpt1, userpt2)
-  XPoint *userpt1, *userpt2;
+long sqwirelen(XPoint *userpt1, XPoint *userpt2)
 {
   long xdist, ydist;
 
@@ -120,8 +113,7 @@ long sqwirelen(userpt1, userpt2)
 /* floating-point version of the above					  */
 /*------------------------------------------------------------------------*/
 
-float fsqwirelen(userpt1, userpt2)
-  XfPoint *userpt1, *userpt2;
+float fsqwirelen(XfPoint *userpt1, XfPoint *userpt2)
 {
   float xdist, ydist;
 
@@ -134,10 +126,9 @@ float fsqwirelen(userpt1, userpt2)
 /* Find absolute distance between two points in user space		  */
 /*------------------------------------------------------------------------*/
 
-int wirelength(userpt1, userpt2)
-  XPoint *userpt1, *userpt2;
+int wirelength(XPoint *userpt1, XPoint *userpt2)
 {
-  unsigned long xdist, ydist;
+  u_long xdist, ydist;
 
   xdist = (long)(userpt2->x) - (long)(userpt1->x);
   ydist = (long)(userpt2->y) - (long)(userpt1->y);
@@ -148,8 +139,7 @@ int wirelength(userpt1, userpt2)
 /* Find the closest (squared) distance from a point to a line		  */
 /*------------------------------------------------------------------------*/
 
-long finddist(linept1, linept2, userpt)
-  XPoint *linept1, *linept2, *userpt;
+long finddist(XPoint *linept1, XPoint *linept2, XPoint *userpt)
 {
    long a, b, c, frac;
    float protod;
@@ -170,8 +160,7 @@ long finddist(linept1, linept2, userpt)
 /* Calculate points for an arc						  */
 /*------------------------------------------------------------------------*/
 
-void calcarc(thearc)
-   arcptr thearc;
+void calcarc(arcptr thearc)
 {
    short idx, thidx;
    int sarc;
@@ -237,9 +226,8 @@ void initsplines()
 /* Compute spline coefficients						  */
 /*------------------------------------------------------------------------*/
 
-void computecoeffs(thespline, ax, bx, cx, ay, by, cy)
-  splineptr thespline;
-  float *ax, *bx, *cx, *ay, *by, *cy;
+void computecoeffs(splineptr thespline, float *ax, float *bx, float *cx,
+	float *ay, float *by, float *cy)
 {
    *cx = 3.0 * (float)(thespline->ctrl[1].x - thespline->ctrl[0].x);
    *bx = 3.0 * (float)(thespline->ctrl[2].x - thespline->ctrl[1].x) - *cx;
@@ -252,8 +240,7 @@ void computecoeffs(thespline, ax, bx, cx, ay, by, cy)
 
 /*------------------------------------------------------------------------*/
 
-void calcspline(thespline)
-   splineptr	thespline;
+void calcspline(splineptr thespline)
 {
    float ax, bx, cx, ay, by, cy;
    short idx;
@@ -271,11 +258,7 @@ void calcspline(thespline)
 /* Find the (x,y) position and tangent rotation of a point on a spline    */
 /*------------------------------------------------------------------------*/
 
-void findsplinepos(thespline, t, retpoint, retrot)
-  splineptr    thespline;
-  float t;
-  XPoint *retpoint;
-  int *retrot;
+void findsplinepos(splineptr thespline, float t, XPoint *retpoint, int *retrot)
 {
    float ax, bx, cx, ay, by, cy;
    float tsq = t * t;
@@ -298,10 +281,7 @@ void findsplinepos(thespline, t, retpoint, retrot)
 /* floating-point version of the above					  */
 /*------------------------------------------------------------------------*/
 
-void ffindsplinepos(thespline, t, retpoint)
-  splineptr    thespline;
-  float t;
-  XfPoint *retpoint;
+void ffindsplinepos(splineptr thespline, float t, XfPoint *retpoint)
 {
    float ax, bx, cx, ay, by, cy;
    float tsq = t * t;
@@ -317,9 +297,7 @@ void ffindsplinepos(thespline, t, retpoint)
 /* fractional distance along the spline of this point.			  */
 /*------------------------------------------------------------------------*/
 
-float findsplinemin(thespline, upoint)
-  splineptr     thespline;
-  XPoint  	*upoint;
+float findsplinemin(splineptr thespline, XPoint *upoint)
 {
    XfPoint 	*spt, flpt, newspt;
    float	minval = 1000000, tval, hval, ndist;
@@ -371,9 +349,7 @@ float findsplinemin(thespline, upoint)
 /* Find closest point of a polygon to the cursor			      */
 /*----------------------------------------------------------------------------*/
 
-short closepoint(curpoly, cursloc)
-  polyptr curpoly;
-  XPoint *cursloc;
+short closepoint(polyptr curpoly, XPoint *cursloc)
 {
    short curdist, mindist;
    XPoint *curpt, *savept; 
@@ -439,9 +415,7 @@ short checkbounds()
 /* Transform X-window coordinate to xcircuit coordinate system		  */
 /*------------------------------------------------------------------------*/
 
-void window_to_user(xw, yw, upt)
-  short 	xw, yw;
-  XPoint	*upt;
+void window_to_user(short xw, short yw, XPoint *upt)
 {
   float tmpx, tmpy;
 
@@ -460,9 +434,7 @@ void window_to_user(xw, yw, upt)
 /* Transform xcircuit coordinate back to X-window coordinate system       */
 /*------------------------------------------------------------------------*/
 
-void user_to_window(upt, wpt)
-  XPoint  upt;
-  XPoint  *wpt;
+void user_to_window(XPoint upt, XPoint *wpt)
 {
   float tmpx, tmpy;
 
@@ -487,8 +459,9 @@ float UTopScale()
    return (float)(sqrt((double)(ctm->a * ctm->a + ctm->d * ctm->d)));
 }
 
-short UTopTransScale(length)
-  float		length;
+/*------------------------------------------------------------------------*/
+
+short UTopTransScale(float length)
 {
    /* 0.5 gives nearent int but unfortunately gives odd results. . .*/
   
@@ -500,8 +473,7 @@ short UTopTransScale(length)
 /*------------------------------------------------------------------------*/
 /* user coordinates to user coordinates version 			  */
 
-void u2u_snap(uvalue)
-  XPoint	*uvalue;
+void u2u_snap(XPoint *uvalue)
 {
    float tmpx, tmpy;
    float tmpix, tmpiy;
@@ -531,10 +503,9 @@ void u2u_snap(uvalue)
 
 /*------------------------------------------------------------------------*/
 /* window coordinates to user coordinates version 			  */
+/*------------------------------------------------------------------------*/
 
-void snap(valuex, valuey, returnpt)
-  int 		valuex, valuey;
-  XPoint 	*returnpt;
+void snap(short valuex, short valuey, XPoint *returnpt)
 {
    window_to_user(valuex, valuey, returnpt);  
    u2u_snap(returnpt);
@@ -594,8 +565,7 @@ void UCopyCTM(fctm, tctm)
 /* matrix from a user point to the X11 window				   */
 /*-------------------------------------------------------------------------*/
 
-void UMakeWCTM(ctm)
-  Matrix *ctm;
+void UMakeWCTM(Matrix *ctm)
 {
    ctm->a *= (*areastruct.vscale);
    ctm->b *= (*areastruct.vscale);
@@ -609,11 +579,7 @@ void UMakeWCTM(ctm)
 
 /*------------------------------------------------------------------------*/
 
-void UMultCTM(ctm, position, scale, rotate)
-  Matrix *ctm;
-  XPoint position;
-  short  rotate; 
-  float  scale;
+void UMultCTM(Matrix *ctm, XPoint position, float scale, short rotate)
 {
    float tmpa, tmpb, tmpd, tmpe, ascale;
    float mata, matb, matc;
@@ -646,9 +612,7 @@ void UMultCTM(ctm, position, scale, rotate)
 /* Slanting function x' = x + beta * y, y' = y				  */
 /*------------------------------------------------------------------------*/
 
-void USlantCTM(ctm, beta)
-  Matrix *ctm;
-  float beta;
+void USlantCTM(Matrix *ctm, float beta)
 {
    ctm->b += ctm->a * beta;
    ctm->e += ctm->d * beta;
@@ -656,8 +620,7 @@ void USlantCTM(ctm, beta)
 
 /*------------------------------------------------------------------------*/
 
-void UPreScaleCTM(ctm)
-  Matrix *ctm;
+void UPreScaleCTM(Matrix *ctm)
 {
    if ((ctm->a < 0) || (ctm->a == 0 && ((ctm->b * ctm->d) < 0))) {
       ctm->a = -ctm->a;
@@ -698,11 +661,7 @@ short flipadjust(short justify)
 
 /*------------------------------------------------------------------------*/
 
-void UPreMultCTM(ctm, position, scale, rotate)
-  Matrix *ctm;
-  XPoint position;
-  short rotate;
-  float scale;
+void UPreMultCTM(Matrix *ctm, XPoint position, float scale, short rotate)
 {
    float tmpa, tmpb, tmpd, tmpe, ascale;
    float mata, matd;
@@ -733,18 +692,17 @@ void UPreMultCTM(ctm, position, scale, rotate)
 
 /*------------------------------------------------------------------------*/
 
-void UTransformbyCTM(ctm, ipoints, points, number)
-  Matrix *ctm;
-  XPoint *ipoints, *points;
-  short  number;
+void UTransformbyCTM(Matrix *ctm, XPoint *ipoints, XPoint *points, short number)
 {
    pointlist current, ptptr = points;
+   short tmpx;
 
    for (current = ipoints; current < ipoints + number; current++, ptptr++) {
-      ptptr->x = (short)(ctm->a * (float)current->x + ctm->b * (float)current->y
+      tmpx = (short)(ctm->a * (float)current->x + ctm->b * (float)current->y
 		 + ctm->c);
       ptptr->y = (short)(ctm->d * (float)current->x + ctm->e * (float)current->y
 		 + ctm->f);
+      ptptr->x = tmpx;
    }
 }
   
@@ -753,15 +711,10 @@ void UTransformbyCTM(ctm, ipoints, points, number)
 /* is for calculation of Bezier curve internal points.			  */
 /*------------------------------------------------------------------------*/
 
-void UfTransformbyCTM(ctm, fpoints, points, number)
-  Matrix *ctm;
-  XfPoint *fpoints;
-  XPoint *points;
-  short number;
+void UfTransformbyCTM(Matrix *ctm, XfPoint *fpoints, XPoint *points, short number)
 {
    fpointlist current;
    pointlist new = points;
-   short tmpx;
 
    for (current = fpoints; current < fpoints + number; current++, new++) {
       new->x = (short)(ctm->a * current->x + ctm->b * current->y + ctm->c);
@@ -771,22 +724,38 @@ void UfTransformbyCTM(ctm, fpoints, points, number)
 
 /*------------------------------------------------------------------------*/
 
-void UPushCTM()
+void UPopCTM()
 {
-   areastruct.MatStackDepth++;
-   areastruct.MatStack = (Matrix *)realloc(areastruct.MatStack,
-		areastruct.MatStackDepth * sizeof(Matrix));
-   UCopyCTM(DCTM2, DCTM);
+   Matrixptr lastmatrix;
+
+   if (areastruct.MatStack == NULL) {
+      Wprintf("Matrix stack pop error");
+      return;
+   }
+   lastmatrix = areastruct.MatStack->nextmatrix;
+   free(areastruct.MatStack);
+   areastruct.MatStack = lastmatrix;
 }
 
 /*------------------------------------------------------------------------*/
-/* UPopCTM()   defined as macro in xcircuit.h				  */
+
+void UPushCTM()
+{
+   Matrixptr newmatrix;
+
+   newmatrix = (Matrixptr)malloc(sizeof(Matrix));
+   if (areastruct.MatStack == NULL)
+      UResetCTM(newmatrix);
+   else
+      UCopyCTM(areastruct.MatStack, newmatrix); 
+   newmatrix->nextmatrix = areastruct.MatStack;
+   areastruct.MatStack = newmatrix;
+}
+
 /*------------------------------------------------------------------------*/
 
-void UTransformPoints(points, newpoints, number, atpt, scale, rotate)
-  XPoint       *points, *newpoints, atpt;
-  short        number, rotate;
-  float        scale;
+void UTransformPoints(XPoint *points, XPoint *newpoints, short number,
+	XPoint atpt, float scale, short rotate)
 {
    Matrix LCTM;
  
@@ -799,10 +768,8 @@ void UTransformPoints(points, newpoints, number, atpt, scale, rotate)
 /* Transform points inward to next hierarchical level */
 /*----------------------------------------------------*/
 
-void InvTransformPoints(points, newpoints, number, atpt, scale, rotate)
-  XPoint       *points, *newpoints, atpt;
-  short        number, rotate;
-  float        scale;
+void InvTransformPoints(XPoint *points, XPoint *newpoints, short number,
+	XPoint atpt, float scale, short rotate)
 {
    Matrix LCTM;
  
@@ -816,9 +783,7 @@ void InvTransformPoints(points, newpoints, number, atpt, scale, rotate)
 /* Translate wire coords to force wire to horizontal or vertical position */
 /*------------------------------------------------------------------------*/
 
-void manhattanize(pospt, newwire)
-  XPoint	*pospt;
-  polyptr	newwire;
+void manhattanize(XPoint *pospt, polyptr newwire)
 {
    short deltax, deltay;
    XPoint *tpoint = newwire->points + newwire->number - 2;
@@ -833,8 +798,7 @@ void manhattanize(pospt, newwire)
 /* Bounding box calculation routines					*/
 /*----------------------------------------------------------------------*/
 
-void bboxcalc(testval, lowerval, upperval)
-  short	testval, *lowerval, *upperval;
+void bboxcalc(short testval, short *lowerval, short *upperval)
 {
    if (testval < *lowerval) *lowerval = testval;
    if (testval > *upperval) *upperval = testval;
@@ -844,9 +808,8 @@ void bboxcalc(testval, lowerval, upperval)
 /* Bounding box calculation for elements which can be part of a path	*/
 /*----------------------------------------------------------------------*/
 
-calcextents(bboxgen, llx, lly, urx, ury)
-  genericptr *bboxgen;
-  short *llx, *lly, *urx, *ury;
+void calcextents(genericptr *bboxgen, short *llx, short *lly, 
+	short *urx, short *ury)
 {
    switch ((*bboxgen)->type) {
       case(POLYGON): {
@@ -894,7 +857,7 @@ void calcbboxsingle(genericptr *bboxgen, short *llx, short *lly, short *urx,
 									)
 {
    XPoint points[4], npoints[4];
-   short i, j;
+   short j;
 
    /* For each screen element, compute the extents and revise bounding	*/
    /* box points, if necessary. 					*/
@@ -929,7 +892,7 @@ void calcbboxsingle(genericptr *bboxgen, short *llx, short *lly, short *urx,
       case(LABEL):
 	 {
 	    TextExtents tmpext;
-            tmpext = ULength(TOLABEL(bboxgen)->string, 0.0, 0, NULL);
+            tmpext = ULength(TOLABEL(bboxgen)->string, NORMINST, 0.0, 0, NULL);
             points[0].x = points[1].x = (TOLABEL(bboxgen)->justify & NOTLEFT ? 
 	       (TOLABEL(bboxgen)->justify & RIGHT ? -tmpext.width :
 		-tmpext.width / 2) : 0);
@@ -979,7 +942,6 @@ void calcbboxsingle(genericptr *bboxgen, short *llx, short *lly, short *urx,
 Boolean object_in_library(short libnum, objectptr thisobject)
 {
    short i;
-   objectptr lobj;
 
    for (i = 0; i < xobjs.userlibs[libnum].number; i++) {
       if (*(xobjs.userlibs[libnum].library + i) == thisobject)
@@ -1029,7 +991,7 @@ void updatepagebounds(objectptr thisobject)
       for (i = 0; i < xobjs.pages; i++)
          if (xobjs.pagelist[i]->pageobj != NULL)
             if ((j = find_object(xobjs.pagelist[i]->pageobj, thisobject)) >= 0) {
-	       calcbboxvalues(xobjs.pagelist[i]->pageobj, (uchar)0,
+	       calcbboxvalues(xobjs.pagelist[i]->pageobj, (u_char)0,
 		       (genericptr *)(xobjs.pagelist[i]->pageobj->plist + j));
 	       updatepagelib(PAGELIB, i);
 	    }
@@ -1046,7 +1008,7 @@ void updatepagebounds(objectptr thisobject)
 
 void calcbbox(objectptr bobj)
 {
-   calcbboxvalues(bobj, (uchar)0, (genericptr *)NULL);
+   calcbboxvalues(bobj, (u_char)0, (genericptr *)NULL);
    if (bobj == objectdata)
       updatepagebounds(objectdata);
 }
@@ -1064,7 +1026,7 @@ void calcbboxparam(objectptr bobj, int mode)
 {
    short i, j;
 
-   calcbboxvalues(bobj, (uchar)0, (genericptr *)NULL);
+   calcbboxvalues(bobj, (u_char)0, (genericptr *)NULL);
 
    /* this is a modified version of updatepagebounds() 	*/
    /* mode == 0:  change bounds on pagelib and the page	*/
@@ -1072,8 +1034,8 @@ void calcbboxparam(objectptr bobj, int mode)
 
    if (mode == 0) {
       /* pick up calling object and instance from the edit stack */
-      objectptr pageobj = (pushlist + pushes - 1)->thisobject;
-      objinstptr pinst = (pushlist + pushes - 1)->thisinst;
+      objectptr pageobj = pushlist->thisobject;
+      objinstptr pinst = NORMINST;
 
       for (j = 0; j < pageobj->parts; j++)
 	 if (*(pageobj->plist + j) == (genericptr)pinst) break;
@@ -1081,7 +1043,7 @@ void calcbboxparam(objectptr bobj, int mode)
 	 fprintf(stderr, "Error: Calling page does not contain expected objinst!\n");
 	 return;
       }
-      calcbboxvalues(pageobj, (uchar)0, (genericptr *)(pageobj->plist + j));
+      calcbboxvalues(pageobj, (u_char)0, (genericptr *)(pageobj->plist + j));
       for (i = 0; i < xobjs.pages; i++)
 	 if (xobjs.pagelist[i]->pageobj == pageobj) {
             updatepagelib(PAGELIB, i);
@@ -1109,8 +1071,8 @@ void calcbboxparam(objectptr bobj, int mode)
 	       /* parameter value is the default value which was just changed.	*/
 	       /* However, then any non-default values may contain the wrong	*/
 	       /* substitutions.						*/
-	       if (cinst->num_params == 0) {
-	          calcbboxvalues(xobjs.pagelist[i]->pageobj, (uchar)0,
+	       if (cinst->thisobject->num_params == 0) {
+	          calcbboxvalues(xobjs.pagelist[i]->pageobj, (u_char)0,
 		       (genericptr *)(xobjs.pagelist[i]->pageobj->plist + j));
 	          updatepagelib(PAGELIB, i);
 	       }
@@ -1129,7 +1091,7 @@ void calcbboxparam(objectptr bobj, int mode)
 
 void singlebbox(objectptr localdata, genericptr *gelem)
 {
-   calcbboxvalues(localdata, (uchar)0, (genericptr *)gelem);
+   calcbboxvalues(localdata, (u_char)0, (genericptr *)gelem);
    updatepagebounds(localdata);
 }
 
@@ -1151,7 +1113,7 @@ void calcbboxselect()
 /* Update Bounding box on an object or single element		*/
 /*--------------------------------------------------------------*/
 
-void calcbboxvalues(objectptr localdata, uchar mode, genericptr *newelement)
+void calcbboxvalues(objectptr localdata, u_char mode, genericptr *newelement)
 {
    genericptr *bboxgen;
    short llx, lly, urx, ury;
@@ -1258,9 +1220,9 @@ void centerview(objectptr localdata)
 /* Refresh the window and scrollbars and write the page name */
 /*-----------------------------------------------------------*/
 
-XtCallbackProc refresh(Widget bw, caddr_t clientdata, caddr_t calldata)
+void refresh(Widget bw, caddr_t clientdata, caddr_t calldata)
 {
-   drawarea(NULL, objectdata, NULL);
+   drawarea(NULL, NULL, NULL);
    drawhbar(areastruct.scrollbarh, NULL, NULL);
    drawvbar(areastruct.scrollbarv, NULL, NULL);
    printname(objectdata);
@@ -1270,9 +1232,7 @@ XtCallbackProc refresh(Widget bw, caddr_t clientdata, caddr_t calldata)
 /* Center the current page in the viewing window	*/
 /*------------------------------------------------------*/
 
-void zoomview(w, clientdata, calldata)
-  Widget 	 w;
-  caddr_t	 clientdata, calldata;
+void zoomview(Widget w, caddr_t clientdata, caddr_t calldata)
 {
    if (eventmode == NORMAL_MODE || eventmode == COPY2_MODE ||
 	 eventmode == PUSH_MODE || eventmode == PRESS_MODE ||
@@ -1291,9 +1251,7 @@ void zoomview(w, clientdata, calldata)
 /* Basic X Graphics Routines in the User coordinate system */
 /*---------------------------------------------------------*/
 
-void UDrawSimpleLine(localdata, pt1, pt2)
-  objinstptr	localdata;
-  XPoint	*pt1, *pt2;
+void UDrawSimpleLine(objinstptr localdata, XPoint *pt1, XPoint *pt2)
 {
    XPoint newpt1, newpt2;
 
@@ -1305,9 +1263,7 @@ void UDrawSimpleLine(localdata, pt1, pt2)
 
 /*-------------------------------------------------------------------------*/
 
-void UDrawLine(localdata, pt1, pt2)
-  objinstptr	localdata;
-  XPoint	*pt1, *pt2;
+void UDrawLine(objinstptr localdata, XPoint *pt1, XPoint *pt2)
 {
    short tmpwidth = UTopTransScale(xobjs.pagelist[areastruct.page]->wirewidth);
 
@@ -1320,8 +1276,7 @@ void UDrawLine(localdata, pt1, pt2)
 /* Add "X" at string origin						   */
 /*-------------------------------------------------------------------------*/
 
-void UDrawX(curlabel)
-  labelptr curlabel;
+void UDrawX(labelptr curlabel)
 {
    XPoint upt;
 
@@ -1357,11 +1312,9 @@ void pinadjust (short justify, short *xpoint, short *ypoint, short dir)
 /*   tpos (2nd parameter)						*/
 /*----------------------------------------------------------------------*/
 
-void UDrawTextLine(curlabel, tpos)
-   labelptr	curlabel;
-   short	tpos;
+void UDrawTextLine(labelptr curlabel, short tpos)
 {
-   XPoint  upt, points[2]; /* top and bottom of text cursor line */
+   XPoint  points[2]; /* top and bottom of text cursor line */
    short   xdist, xbase, tmpjust;
    TextExtents tmpext;
 
@@ -1374,10 +1327,10 @@ void UDrawTextLine(curlabel, tpos)
    XSetFunction(dpy, areastruct.gc, GXxor);
    XSetForeground(dpy, areastruct.gc, AUXCOLOR ^ BACKGROUND);
 
-   tmpext = ULength(curlabel->string, 0.0, tpos, NULL);
+   tmpext = ULength(curlabel->string, NORMINST, 0.0, tpos, NULL);
    xdist = tmpext.width;
    xbase = tmpext.base;
-   tmpext = ULength(curlabel->string, 0.0, 0, NULL);
+   tmpext = ULength(curlabel->string, NORMINST, 0.0, 0, NULL);
 
    points[0].x = (tmpjust & NOTLEFT ?
         (tmpjust & RIGHT ? -tmpext.width : -tmpext.width >> 1) : 0)
@@ -1407,8 +1360,7 @@ void UDrawTextLine(curlabel, tpos)
 /* Draw lines for editing text when multiple characters are chosen */
 /*-----------------------------------------------------------------*/
 
-void UDrawTLine(curlabel)
-   labelptr	curlabel;
+void UDrawTLine(labelptr curlabel)
 {
    UDrawTextLine(curlabel, textpos);
    if ((textend > 0) && (textend < textpos)) {
@@ -1420,8 +1372,7 @@ void UDrawTLine(curlabel)
 /* Draw an X		*/
 /*----------------------*/
 
-void UDrawXLine(opt, cpt)
-  XPoint	opt, cpt;
+void UDrawXLine(XPoint opt, XPoint cpt)
 {
    XPoint upt, vpt;
 
@@ -1443,8 +1394,8 @@ void UDrawXLine(opt, cpt)
 }
 
 /*-------------------------------------------------------------------------*/
-void UDrawBox(origin, corner)
-  XPoint	origin, corner;
+
+void UDrawBox(XPoint origin, XPoint corner)
 {
    XPoint	worig, wcorn;
 
@@ -1484,15 +1435,11 @@ void UDrawBBox(objectptr bbobj)
 /* Fill and/or draw a border around the stroking path			   */
 /*-------------------------------------------------------------------------*/
 
-void strokepath(pathlist, number, style, width)
-  XPoint *pathlist;
-  short  number;
-  short  style;
-  float  width;
+void strokepath(XPoint *pathlist, short number, short style, float width)
 {
    char         solidpart;
    char         dashstring[3];
-   short        tmpwidth, i;
+   short        tmpwidth;
 
    tmpwidth = max(1, UTopTransScale(xobjs.pagelist[areastruct.page]->wirewidth *
 	width));
@@ -1541,9 +1488,7 @@ void strokepath(pathlist, number, style, width)
 
 /*-------------------------------------------------------------------------*/
 
-void makesplinepath(thespline, pathlist)
-  splineptr      thespline;
-  XPoint	*pathlist;
+void makesplinepath(splineptr thespline, XPoint *pathlist)
 {
    XPoint *tmpptr = pathlist;
 
@@ -1554,9 +1499,7 @@ void makesplinepath(thespline, pathlist)
 
 /*-------------------------------------------------------------------------*/
 
-void UDrawSpline(localdata, thespline)
-  objinstptr     localdata;
-  splineptr      thespline;
+void UDrawSpline(objinstptr localdata, splineptr thespline)
 {
    XPoint       tmppoints[SPLINESEGS];
 
@@ -1566,9 +1509,7 @@ void UDrawSpline(localdata, thespline)
 
 /*-------------------------------------------------------------------------*/
 
-void UDrawEditSpline(localdata, thespline)
-  objinstptr     localdata;
-  splineptr      thespline;
+void UDrawEditSpline(objinstptr localdata, splineptr thespline)
 {
    UDrawSpline(localdata, thespline);
    UDrawXLine(thespline->ctrl[0], thespline->ctrl[1]);  
@@ -1577,9 +1518,7 @@ void UDrawEditSpline(localdata, thespline)
 
 /*-------------------------------------------------------------------------*/
 
-void UDrawPolygon(localdata, thepoly)
-  objinstptr     localdata;
-  polyptr	 thepoly;
+void UDrawPolygon(objinstptr localdata, polyptr thepoly)
 {
    XPoint *tmppoints = (pointlist) malloc(thepoly->number * sizeof(XPoint));
 
@@ -1590,9 +1529,7 @@ void UDrawPolygon(localdata, thepoly)
 
 /*-------------------------------------------------------------------------*/
 
-void UDrawArc(localdata, thearc)
-  objinstptr	localdata;
-  arcptr	thearc;
+void UDrawArc(objinstptr localdata, arcptr thearc)
 {
    XPoint  tmppoints[RSTEPS + 2];
 
@@ -1602,9 +1539,7 @@ void UDrawArc(localdata, thearc)
 
 /*-------------------------------------------------------------------------*/
 
-void UDrawPath(localdata, thepath)
-  objinstptr	localdata;
-  pathptr	thepath;
+void UDrawPath(objinstptr localdata, pathptr thepath)
 {
    XPoint	*tmppoints = (pointlist) malloc(sizeof(XPoint));
    genericptr	*genpath;
@@ -1644,32 +1579,29 @@ void UDrawPath(localdata, thepath)
 }
 
 /*----------------------------------------------------------------------*/
-/* Main recursive object drawing routine				*/
+/* Main recursive object drawing routine.				*/
+/*    level is the level of recursion 					*/
+/*    passcolor is the inherited color value passed to object		*/
 /*----------------------------------------------------------------------*/
 
-void UDrawObject(localdata, theobject, level, passcolor)
-  objinstptr	localdata;
-  objectptr	theobject;
-  short		level;		/* level of recursion */
-  int		passcolor;	/* inherited color value passed to object */
+void UDrawObject(objinstptr localdata, objectptr theobject, short level,
+	int passcolor)
 {
    genericptr	*areagen;
-   short	*selectobj;
    short	tmpwidth;
    int		defaultcolor = passcolor;
    int		curcolor = passcolor;
    XPoint 	bboxin, bboxout[2];
    Boolean	xm, ym;
-
-   /* replace parameters (if any) from the instance into the object */
-   if (level != 0) psubstitute(localdata);
+   objectpair	*newhierarchy, *lasthierarchy;
 
    /* add self to hierarchy list */
 
-   areastruct.hlevels++;
-   areastruct.hierarchy = (objinstptr *) realloc (areastruct.hierarchy,
-      (areastruct.hlevels + 1) * sizeof(objinstptr));
-   *(areastruct.hierarchy + areastruct.hlevels) = localdata;
+   newhierarchy = (objectpair *)malloc(sizeof(objectpair));
+   newhierarchy->nextpair = areastruct.hierarchy;
+   areastruct.hierarchy = newhierarchy;
+   newhierarchy->thisinst = localdata;
+   newhierarchy->thisobject = theobject;
 
    UPushCTM();
    UPreMultCTM(DCTM, localdata->position, localdata->scale,
@@ -1741,13 +1673,16 @@ void UDrawObject(localdata, theobject, level, passcolor)
 
      /* restore the color passed to the object, if different from current color */
 
-     if ((defaultcolor != DOFORALL) && (passcolor != curcolor))
+     if ((defaultcolor != DOFORALL) && (passcolor != curcolor)) {
 	XTopSetForeground(passcolor);
+     }
    }
 
    /* pop the hierarchy stack */
 
-   areastruct.hlevels--;
+   lasthierarchy = areastruct.hierarchy->nextpair;
+   free(areastruct.hierarchy);
+   areastruct.hierarchy = lasthierarchy;
    UPopCTM();
 }
 
