@@ -1,6 +1,6 @@
 /*-------------------------------------------------------------------------*/
 /* libraries.c --- xcircuit routines for the builtin and user libraries    */
-/* Copyright (c) 2000  Tim Edwards, Johns Hopkins University        	   */
+/* Copyright (c) 2001  Tim Edwards, Johns Hopkins University        	   */
 /*-------------------------------------------------------------------------*/
 
 /*-------------------------------------------------------------------------*/
@@ -8,8 +8,11 @@
 /*-------------------------------------------------------------------------*/
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
+#if !defined(__DARWIN__)
 #include <malloc.h>
+#endif
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
@@ -24,6 +27,11 @@
 #include "colordefs.h"
 #include "xcircuit.h"
 
+/*----------------------------------------------------------------------*/
+/* Function prototype declarations                                      */
+/*----------------------------------------------------------------------*/
+#include "prototypes.h"
+
 /*-------------------------------------------------------------------------*/
 /* Global Variable definitions						   */
 /*-------------------------------------------------------------------------*/
@@ -37,19 +45,10 @@ extern Globaldata xobjs;
 extern Clientdata areastruct;
 extern char _STR[150];
 extern objectpair *pushlist;
-extern short pushes;
 extern short fontcount;
 extern fontinfo *fonts;
 
-/*------------------------------------------------------------------------*/
-/* Declarations of functions defined externally to events.c	 	  */
-/*------------------------------------------------------------------------*/
-
-extern void drawarea(), drag(), newpage(), setpage();
-extern short *allocselect();
-void composelib(short), composepagelib(short), changecat();
-void computespacing(short, int *, int *, int *, int *);
-XtCallbackProc startcatalog(Widget, void *, caddr_t);
+/*---------------------------------------------------------*/
 
 #ifdef SCHEMA
 #define _lowerleft lleft2
@@ -180,10 +179,7 @@ int is_library(objectptr thisobj)
 /* ButtonPress handler during page catalog viewing mode */
 /*------------------------------------------------------*/
 
-XtEventHandler pagecatbutton(w, nulldata, event)
-  Widget w;
-  caddr_t nulldata;
-  XButtonEvent *event;
+void pagecatbutton(XButtonEvent *event)
 {
    int bpage;
    short mode;
@@ -202,13 +198,13 @@ XtEventHandler pagecatbutton(w, nulldata, event)
 	       /* using changepage() allows use of new page for schematic */
 	       changepage(bpage);
 	       /* associate the new schematic */
-	       schemassoc(objectdata, (*(pushlist + pushes - 1)).thisobject);
+	       schemassoc(objectdata, pushlist->thisobject);
 	       /* pop back to calling (symbol) page */
 	       catreturn();
 	       eventmode = PRESS_MODE;
 	    }
 	    else {
-	       startcatalog(w, Number(LIBRARY + bpage), NULL);
+	       startcatalog(NULL, LIBRARY + bpage, NULL);
 	    }
 	    return;
          }
@@ -224,7 +220,7 @@ XtEventHandler pagecatbutton(w, nulldata, event)
       	       eventmode = PRESS_MODE;
 	    }
 	    else {
-	       startcatalog(w, Number(LIBRARY + bpage), NULL);
+	       startcatalog(NULL, LIBRARY + bpage, NULL);
 	    }
 	    return;
 	 }
@@ -245,8 +241,8 @@ XtEventHandler pagecatbutton(w, nulldata, event)
 /* representing an entire page in the page directory.				*/
 /*------------------------------------------------------------------------------*/
 
-pageinstpos(short mode, short tpage, objinstptr drawinst, int gxsize, int gysize,
-		int xdel, int ydel)
+void pageinstpos(short mode, short tpage, objinstptr drawinst, int gxsize,
+	int gysize, int xdel, int ydel)
 {
    objectptr libobj = drawinst->thisobject;
    float scalex, scaley;
@@ -292,8 +288,7 @@ objinstptr makeinstofpage(objectptr libinst, objectptr libobj)
    NEW_OBJINST(drawinst, libinst);
    (*drawinst)->color = DEFAULTCOLOR;
    (*drawinst)->rotation = 1;
-   (*drawinst)->params = NULL;
-   (*drawinst)->num_params = (uchar)0;
+   (*drawinst)->params = NULL;		/* values are the defaults */
    (*drawinst)->thisobject = libobj;
    libinst->parts++;
    return *drawinst;
@@ -322,13 +317,11 @@ void computespacing(short mode, int *gxsize, int *gysize, int *xdel, int *ydel)
 
 void composepagelib(short mode)
 {
-   labelptr *drawname;
    objinstptr drawinst;
    objectptr libobj, libinst = xobjs.libtop[mode];
-   short libobjects, i;
+   short i;
    polyptr *drawbox;
    pointlist pointptr;
-   XPoint dpt;
    int bbuf, xdel, ydel, gxsize, gysize;
    int pages = (mode == PAGELIB) ? xobjs.pages : xobjs.numlibs;
 
@@ -488,7 +481,7 @@ void pagecatmove(XKeyEvent *event)
 
    objectdeselect();
    composelib(PAGELIB);
-   drawarea(NULL, objectdata, NULL);
+   drawarea(NULL, NULL, NULL);
 }
 
 /*-----------------------------------------*/
@@ -562,7 +555,6 @@ void composelib(short mode)
       (*drawinst)->rotation = 1;
       (*drawinst)->scale = 1.0;
       (*drawinst)->params = NULL;	/* library object has all default values */
-      (*drawinst)->num_params = (uchar)0;
       (*drawinst)->thisobject = (*libobj);
 
       (*drawinst)->position.x = xpos - (*libobj)->_lowerleft.x;
@@ -574,16 +566,20 @@ void composelib(short mode)
 	   ((170 - (*libobj)->_height) >> 1);
 
       if (fval < fontcount) {
+	 stringpart *strptr;
+
          drawname = (labelptr *)libinst->plist + (libinst->parts++) + visobjects;
          *drawname = (labelptr) malloc(sizeof(label));      
          (*drawname)->type = LABEL;
+	 labeldefaults(*drawname, False, 0, 0);
+
          (*drawname)->color = DEFAULTCOLOR;
-         (*drawname)->rotation = 1;
          (*drawname)->scale = 0.75;
-         (*drawname)->string = (uchar *) malloc((strlen((*libobj)->name) + 3) *
-	      sizeof(uchar));
-         sprintf((*drawname)->string, "%c%c%s", (char)TEXT_ESC,
-		(char)(FONT_START + fval), (*libobj)->name);
+	 (*drawname)->string->data.font = fval;
+	 strptr = makesegment(&((*drawname)->string), NULL);
+	 strptr->type = TEXT_STRING;
+	 strptr->data.string = (char *) malloc(1 + strlen((*libobj)->name));
+         strcpy(strptr->data.string, (*libobj)->name);
          (*drawname)->justify = TOP | NOTBOTTOM | NOTLEFT;
 #ifdef SCHEMA
          (*drawname)->pin = False;
@@ -621,9 +617,7 @@ void composelib(short mode)
 /*   Object/Page with dependency (if any) returned in "compobjp". */
 /*----------------------------------------------------------------*/
 
-short finddepend(libobj, compobjp)
-  objinstptr libobj;
-  objectptr **compobjp;
+short finddepend(objinstptr libobj, objectptr **compobjp)
 {
    genericptr *testobj;
    short page, i, j;
@@ -663,12 +657,11 @@ short finddepend(libobj, compobjp)
 /* "Hide" an object (must have a dependency or else it disappears)*/
 /*----------------------------------------------------------------*/
 
-cathide()
+void cathide()
 {
    short *newselect;
    objectptr *compobj;
    objinstptr libobj;
-   short i;
 
    if (areastruct.selects == 0) return;
 
@@ -696,14 +689,14 @@ cathide()
    else
       composelib(USERLIB);
 
-   drawarea(NULL, objectdata, NULL);
+   drawarea(NULL, NULL, NULL);
 }
 
 /*----------------------------------------------------------------*/
 /* Delete an object from the library if there are no dependencies */
 /*----------------------------------------------------------------*/
 
-catdelete()
+void catdelete()
 {
    short *newselect;
    genericptr *testobj, *tobj;
@@ -770,7 +763,7 @@ catdelete()
    else
       composelib(USERLIB);
 
-   drawarea(NULL, objectdata, NULL);
+   drawarea(NULL, NULL, NULL);
 }
 
 /*--------------------------------------------------------*/
@@ -895,7 +888,7 @@ void catmove(XKeyEvent *event)
    else
       composelib(USERLIB);
 
-   drawarea(NULL, objectdata, NULL);
+   drawarea(NULL, NULL, NULL);
 }
 
 /*--------------------------------------------------------*/
@@ -949,7 +942,7 @@ void copycat()
 
       (*newobj)->schemtype = oldobj->schemtype;
       (*newobj)->netlist = NULL;
-      (*newobj)->pinlist = NULL;
+      (*newobj)->portlist = NULL;
       (*newobj)->calllist = NULL;
       (*newobj)->traversed = 0;
 #endif
@@ -1015,7 +1008,6 @@ void copycat()
 
 	    case(SPLINE): {
 	       register splineptr *nspl, cspl;
-	       register int j;
 
 	       cspl = TOSPLINE(oldobj->plist + i);
 	       NEW_SPLINE(nspl, (*newobj));
@@ -1033,9 +1025,7 @@ void copycat()
 	       (*nlabel)->position.y = clabel->position.y;
 	       (*nlabel)->scale = clabel->scale;
 	       (*nlabel)->justify = clabel->justify;
-	       (*nlabel)->string = (char *)malloc((strlen(clabel->string) + 1) *
-		      sizeof(char));
-	       strcpy((*nlabel)->string, clabel->string);
+	       (*nlabel)->string = stringcopy(clabel->string);
 #ifdef SCHEMA
 	       (*nlabel)->pin = clabel->pin;
 #endif
@@ -1055,7 +1045,6 @@ void copycat()
 
 	       /* Library instance's parameter list is always default (null) */
 	       (*ninst)->params = NULL;
-	       (*ninst)->num_params = (uchar)0;
 	      
 #ifdef SCHEMA
 	       /* here---deal with attached schematics */
@@ -1070,18 +1059,15 @@ void copycat()
    objectdeselect();
 
    if (objectdata == xobjs.libtop[USERLIB])
-      drawarea(NULL, objectdata, NULL);
-   else startcatalog(NULL, Number(USERLIB), NULL);
+      drawarea(NULL, NULL, NULL);
+   else startcatalog(NULL, USERLIB, NULL);
 }
 
 /*--------------------------------------------------------*/
 /* ButtonPress handler during normal catalog viewing mode */
 /*--------------------------------------------------------*/
 
-XtEventHandler catbutton(w, mode, event)
-  Widget w;
-  caddr_t mode;
-  XButtonEvent *event;
+void catbutton(u_int mode, XButtonEvent *event)
 {
    short *newselect;
    objinstptr *newobject, *libobj;
@@ -1092,7 +1078,7 @@ XtEventHandler catbutton(w, mode, event)
    /* Make catbutton() a wrapper for pagecatbutton() */
 
    if (is_library(objectdata) < 0) {
-      pagecatbutton(w, mode, event);
+      pagecatbutton(event);
       return;
    }
 
@@ -1124,8 +1110,9 @@ XtEventHandler catbutton(w, mode, event)
 	       if (eventmode == ASSOC_MODE) {
 
 	          /* revert to old page */
-	          objectdata = (pushes == 0) ? xobjs.pagelist[areastruct.page]->pageobj
-			   : (*(pushlist + pushes - 1)).thisobject;
+	          objectdata = (pushlist == NULL) ?
+			   xobjs.pagelist[areastruct.page]->pageobj
+			   : pushlist->thisobject;
 		  /* associate the new object */
 		  schemassoc(objectdata, (*libobj)->thisobject); 
    	          setpage();
@@ -1138,8 +1125,9 @@ XtEventHandler catbutton(w, mode, event)
 
 	          /* revert to old page */
 
-	          objectdata = (pushes == 0) ? xobjs.pagelist[areastruct.page]->pageobj
-			   : (*(pushlist + pushes - 1)).thisobject;
+	          objectdata = (pushlist == NULL) ?
+			   xobjs.pagelist[areastruct.page]->pageobj
+			   : pushlist->thisobject;
 
 	          /* retrieve drawing window state and set position of object to
 	             be correct in reference to that window */
@@ -1170,7 +1158,6 @@ XtEventHandler catbutton(w, mode, event)
 		     (*newobject)->position.y = TOOBJINST(libpage->plist +
 		            *newselect)->position.y + ydiff;
 		     (*newobject)->params = NULL;
-		     (*newobject)->num_params = (uchar)0;
 	             u2u_snap(&((*newobject)->position));
 		     *newselect = (short)(newobject - (objinstptr *)objectdata->plist);
 		     if ((*newobject)->thisobject == (*libobj)->thisobject)
@@ -1189,7 +1176,6 @@ XtEventHandler catbutton(w, mode, event)
                      (*newobject)->position.x = areastruct.save.x;
 	             (*newobject)->position.y = areastruct.save.y; 
 		     (*newobject)->params = NULL;
-		     (*newobject)->num_params = (uchar)0;
 
 	             /* add this object to the list of selected items */
 
@@ -1203,13 +1189,13 @@ XtEventHandler catbutton(w, mode, event)
 
                      XDefineCursor(dpy, win, COPYCURSOR);
                      eventmode = COPY2_MODE;
-                     XtAddEventHandler(areastruct.area, PointerMotionMask, False, drag,
-                          NULL);
+                     XtAddEventHandler(areastruct.area, PointerMotionMask, False,
+			  (XtEventHandler)drag, NULL);
 	          }
 	          else {
                      eventmode = PRESS_MODE;
                      XtAddEventHandler(areastruct.area, Button1MotionMask, False,
-		          drag, NULL);
+		          (XtEventHandler)drag, NULL);
 	          }
                   catreturn();
                }
@@ -1271,19 +1257,18 @@ void changecat()
 	 return;
       }
    }
-   startcatalog(NULL, Number(j + LIBRARY), NULL);
+   startcatalog(NULL, j + LIBRARY, NULL);
 }
 
 /*--------------------------------------*/
 /* Begin catalog viewing mode		*/
 /*--------------------------------------*/
 
-XtCallbackProc startcatalog(Widget w, void *libmod, caddr_t nulldata)
+void startcatalog(Widget w, u_int libmod, caddr_t nulldata)
 {
-   int lmod = (int)libmod;
-   if ((xobjs.libtop[lmod] == NULL) || (objectdata == xobjs.libtop[lmod])) return;
+   if ((xobjs.libtop[libmod] == NULL) || (objectdata == xobjs.libtop[libmod])) return;
 
-   if (lmod == FONTLIB) {
+   if (libmod == FONTLIB) {
       XDefineCursor (dpy, win, CROSS);
       if (eventmode == TEXT2_MODE)
          eventmode = FONTCAT_MODE;
@@ -1295,7 +1280,7 @@ XtCallbackProc startcatalog(Widget w, void *libmod, caddr_t nulldata)
       XDefineCursor (dpy, win, CROSS);
    }
 #endif
-   else if (lmod == PAGELIB || lmod == LIBLIB) {
+   else if (libmod == PAGELIB || libmod == LIBLIB) {
       XDefineCursor (dpy, win, CROSS);
       eventmode = CATALOG_MODE;
    }
@@ -1308,22 +1293,20 @@ XtCallbackProc startcatalog(Widget w, void *libmod, caddr_t nulldata)
 
 
    if (!(((is_library(objectdata) >= 0) || (objectdata == xobjs.libtop[LIBLIB])
-		|| (objectdata == xobjs.libtop[PAGELIB])) && lmod >= PAGELIB)) {
-      if (pushes == 0)
-         pushlist = (objectpair *) malloc(sizeof(objectpair));
-      else
-         pushlist = (objectpair *) realloc(pushlist,
-            (pushes + 1) * sizeof(objectpair));
-      (*(pushlist + pushes)).thisobject = objectdata;
-      (*(pushlist + pushes)).thisinst = NULL;
-      pushes++;
+		|| (objectdata == xobjs.libtop[PAGELIB])) && libmod >= PAGELIB)) {
+      objectpair *newpush;
+      newpush = (objectpair *) malloc(sizeof(objectpair));
+      newpush->nextpair = pushlist;
+      pushlist = newpush;
+      newpush->thisobject = objectdata;
+      newpush->thisinst = NULL;
    }
 
    /* set library as new object */
 
-   objectdata = xobjs.libtop[lmod];
+   objectdata = xobjs.libtop[libmod];
    setpage();
-   if (lmod == FONTLIB)
+   if (libmod == FONTLIB)
       zoomview(NULL, NULL, NULL);
 
    /* unselect all current selects */
