@@ -1,6 +1,6 @@
 /*-------------------------------------------------------------------------*/
 /* schema.c --- xcircuit routines specific to the schematic capture system */
-/* Copyright (c) 2000  Tim Edwards, Johns Hopkins University        	   */
+/* Copyright (c) 2001  Tim Edwards, Johns Hopkins University        	   */
 /*-------------------------------------------------------------------------*/
 
 /*-------------------------------------------------------------------------*/
@@ -8,8 +8,11 @@
 /*-------------------------------------------------------------------------*/
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
+#if !defined(__DARWIN__)
 #include <malloc.h>
+#endif
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
@@ -20,11 +23,16 @@
 /* Local includes							   */
 /*-------------------------------------------------------------------------*/
 
+#include "xcircuit.h"
 #include "cursors.h"
 #include "colordefs.h"
-#include "xcircuit.h"
 #include "menudep.h"
 
+/*----------------------------------------------------------------------*/
+/* Function prototype declarations                                      */
+/*----------------------------------------------------------------------*/
+#include "prototypes.h"
+
 /*------------------------------------------------------------------------*/
 /* External Variable definitions                                          */
 /*------------------------------------------------------------------------*/
@@ -32,14 +40,11 @@
 extern Globaldata xobjs;
 extern Clientdata areastruct;
 extern short	  eventmode;
-extern short	  pushes;
 extern int	  *appcolors;
 extern Widget     menuwidgets[];
 extern Widget	  netbutton;
-
-XtCallbackProc	  schembutton();
-objectptr	  recurselect();
-short		  *selectobject();
+extern objectpair *pushlist;
+extern char	  _STR[150];
 
 /*-------------------------------------------------------------------------*/
 #ifdef SCHEMA
@@ -50,7 +55,7 @@ extern Widget wsymb, wschema;
 /* Menu calls (procedure wrappers)			  */
 /*--------------------------------------------------------*/
 
-XtCallbackProc callgennet(Widget w, unsigned int mode, caddr_t calldata)
+void callgennet(Widget w, u_int mode, caddr_t calldata)
 {
    switch(mode) {
      case 0:
@@ -69,9 +74,8 @@ XtCallbackProc callgennet(Widget w, unsigned int mode, caddr_t calldata)
 /* Enable schematic capture (set schemon and realize widgets) */
 /*------------------------------------------------------------*/
 
-XtCallbackProc doxschema(Widget w, caddr_t clientdata, caddr_t calldata)
+void doxschema(Widget w, caddr_t clientdata, caddr_t calldata)
 {
-   short i;
    Arg wargs[1];
 
    if (areastruct.schemon) {
@@ -96,16 +100,14 @@ XtCallbackProc doxschema(Widget w, caddr_t clientdata, caddr_t calldata)
    /* Redraw the screen, since this action may effect the presence of	*/
    /* pins and schematic informational labels.				*/
 
-   drawarea(NULL, objectdata, NULL);
+   drawarea(NULL, NULL, NULL);
 }
 
 /*----------------------------------*/
 /* Find connectivity of a selection */
 /*----------------------------------*/
 
-void startconnect(button, clientdata, calldata)
-  Widget       button;
-  caddr_t      clientdata, calldata;
+void startconnect(Widget button, caddr_t clientdata, caddr_t calldata)
 {
    if (eventmode == NORMAL_MODE) {
       Wprintf("Click button1 on wire to see connectivity.");
@@ -113,66 +115,156 @@ void startconnect(button, clientdata, calldata)
    }
 }
 
-/*------------------------------------------------------------------------*/
+/*----------------------------------------------------------------------*/
+/* rconnectivity():  Find electrical connections into a node.		*/
+/* (does recursive search on object heirarchy)				*/
+/*----------------------------------------------------------------------*/
 
-XtCallbackProc connectivity(button, clientdata, calldata)
-  Widget       button;
-  caddr_t      clientdata, calldata;
+void connectivity(Widget button, caddr_t clientdata, caddr_t calldata)
 {
-   polyptr gpoly;
-   int netid;
+   genericptr ggen;
+   int depth, netid;
+   objectpair *seltop, *nextptr;
+   objectptr nettop;
+   char *snew;
+   stringpart *ppin;
 
-   /* Will have to refine recursive selection at a later date. . . */
-   /* objectptr gobj;				*/
-   /* gobj = recurselect(&gpoly, objectdata);	*/
+   seltop = (objectpair *)malloc(sizeof(objectpair));
+   seltop->thisobject = objectdata;
+   seltop->thisinst = NULL;
+   seltop->nextpair = NULL;
 
-   objectselect(POLYGON);
-   if (areastruct.selects != 1) {
-      Wprintf("Must select one polygon to find network connections.");
-   }
-   else {
-      gpoly = SELTOPOLY(areastruct.selectlist);
-      objectdeselect();
-      createnets(True);
-      netid = resolved(objectdata->netlist, gpoly);
-      /* printf(">> Resolving connectivity for net %d\n", netid);  */
-      if (netid != 0)
-         highlightnet(objectdata, netid);
+   /* Determine the netid and the topmost object in which that	*/
+   /* net appears.  Then build the transformation matrix down	*/
+   /* to that object.  highlightnet() should end by popping the */
+   /* entire matrix stack.					*/
+
+   if ((ggen = recurselect(&seltop)) != NULL) {
+      createnets(True, NULL);
+      if ((netid = is_resolved(&ggen, seltop, &nettop)) != 0) {
+         /* printf("Net ID is %d in object %s\n", netid, nettop->name); */
+         depth = pushnetwork(seltop, nettop);
+         /* printf(">> Pushed network %d levels deep\n", i); */
+         nextptr = seltop;
+         while (nextptr->thisobject != nettop)
+	    nextptr = nextptr->nextpair;
+         highlightnet(nettop, nextptr->thisinst, netid);
+
+         /* pop the matrix stack */
+         while (depth-- > 0) 
+	    UPopCTM();
+
+	 /* print the net name to the message window */
+
+	 ppin = nettopin(netid, nettop, NULL);
+         snew = textprint(ppin, NORMINST);
+	 sprintf(_STR, "Network is \"%s\" in %s", snew, nettop->name);
+	 Wprintf(_STR);
+         free(snew);
+      }
       else
-         Wprintf("No network is associated with this polygon");
+	 Wprintf("Selected element is not part of a valid network.");
       destroynets();
    }
+   else 
+      Wprintf("No networks found near the cursor position");
+
+   /* free up linked list */
+
+   while (seltop != NULL) {
+      nextptr = seltop->nextpair;
+      free(seltop);
+      seltop = nextptr;
+   }
 }
 
-/*-----------------------------------------------------------------------*/
-/* Set object type to FUNDAMENTAL if it contains one or more info labels */
-/*-----------------------------------------------------------------------*/
+/*----------------------------------------------------------------------*/
+/* connectivity():  Find electrical connections into a node.		*/
+/* (simple search only on topmost object)				*/
+/*----------------------------------------------------------------------*/
 
-void setobjecttype(objectptr cschem)
+void tconnectivity(Widget button, caddr_t clientdata, caddr_t calldata)
+{
+   genericptr *ggen;
+   int netid;
+   short *csel;
+   objectpair *seltop;
+   objectptr retobj;
+
+   seltop = (objectpair *)malloc(sizeof(objectpair));
+   seltop->thisobject = objectdata;
+   seltop->thisinst = NULL;
+   seltop->nextpair = NULL;
+
+   objectselect(POLYGON | LABEL);
+   drawselects();
+   if (areastruct.selects == 0) {
+      Wprintf("Must select at least one polygon or label to find network connections.");
+   }
+   else {
+      createnets(True, NULL);
+      for (csel = areastruct.selectlist; csel < areastruct.selectlist +
+		areastruct.selects; csel++) {
+         ggen = objectdata->plist + (*csel);
+         netid = is_resolved(ggen, seltop, &retobj);
+         printf(">> Resolving connectivity for net %d\n", netid);
+	 if (retobj != objectdata)
+	    printf(">> Error:  Resolved object is not topmost object!\n");
+         if (netid != 0)
+            highlightnet(retobj, areastruct.topobject, netid);
+         else
+            Wprintf("No network is associated with this polygon");
+      }
+      destroynets();
+   }
+   clearselects();
+
+   free(seltop);
+}
+
+/*--------------------------------------------------------------*/
+/* Set object type to FUNDAMENTAL if it contains one or more	*/
+/* info labels for the given netlist mode.			*/
+/*								*/
+/* Return TRUE if the object has eletrically relevant networks,	*/
+/* FALSE if not.						*/
+/*--------------------------------------------------------------*/
+
+Boolean setobjecttype(objectptr cschem, char *mode)
 {
    genericptr *cgen;
    labelptr clab;
 
-   if (cschem->schemtype == SCHEMATIC
-	|| cschem->symschem != NULL) return;  /* does not apply to these types */
+   /* Apply only to schematic objects */
+   if (cschem->schemtype != SCHEMATIC && cschem->symschem == NULL) {
 
-   if (cschem->schemtype == FUNDAMENTAL) cschem->schemtype = SYMBOL;
-   for (cgen = cschem->plist; cgen < cschem->plist + cschem->parts; cgen++) {
-      if ((*cgen)->type == LABEL) {
-         clab = TOLABEL(cgen);
-	 if (clab->pin == INFO) {
-	    cschem->schemtype = FUNDAMENTAL;
-	    break;
-	 }
+      if (cschem->schemtype == FUNDAMENTAL) cschem->schemtype = SYMBOL;
+      for (cgen = cschem->plist; cgen < cschem->plist + cschem->parts; cgen++) {
+         if ((*cgen)->type == LABEL) {
+            clab = TOLABEL(cgen);
+	    if (clab->pin == INFO) {
+	       if ((mode == NULL) || !textncomp(clab->string, mode, NULL)) {
+	          cschem->schemtype = FUNDAMENTAL;
+	          break;
+	       }
+	    }
+         }
       }
    }
+
+   if ((cschem->symschem != NULL) && (cschem->schemtype == SYMBOL))
+      return False;
+   else if ((cschem->schemtype == TRIVIAL) || (cschem->schemtype == FUNDAMENTAL))
+      return False;
+
+   return True;
 }
 
 /*------------------------------------------------------*/
 /* Pin conversion subroutine for dopintype()		*/
 /*------------------------------------------------------*/
 
-void pinconvert(labelptr thislab, unsigned int mode)
+void pinconvert(labelptr thislab, u_int mode)
 {
    thislab->pin = mode;
    switch (mode) {
@@ -195,7 +287,7 @@ void pinconvert(labelptr thislab, unsigned int mode)
 /* Change a label's type to NORMAL, GLOBAL, INFO, or LOCAL */
 /*---------------------------------------------------------*/
 
-XtCallbackProc dopintype(Widget w, unsigned int mode, caddr_t calldata)
+void dopintype(Widget w, u_int mode, caddr_t calldata)
 {
    short *gsel;
    char typestr[40];
@@ -228,12 +320,12 @@ XtCallbackProc dopintype(Widget w, unsigned int mode, caddr_t calldata)
 	 labelptr glab = SELTOLABEL(gsel);
 	 savetype = glab->pin;
 	 pinconvert(glab, mode);
-	 setobjecttype(objectdata);
+	 setobjecttype(objectdata, NULL);
       }
 
    if (savetype >= 0) {
       objectdeselect();
-      drawarea(NULL, objectdata, NULL);
+      drawarea(NULL, NULL, NULL);
       Wprintf(typestr);
    }
    else {
@@ -320,11 +412,9 @@ void setsymschem()
 /* Find the page number for an object			  */
 /*--------------------------------------------------------*/
 
-int findpageobj(pobj)
-  objectptr pobj;
+int findpageobj(objectptr pobj)
 {
    int tpage;
-   objectptr chkpage;
 
    for (tpage = 0; tpage < xobjs.pages; tpage++) {
       if (xobjs.pagelist[tpage]->pageobj == pobj) return tpage;
@@ -338,9 +428,7 @@ int findpageobj(pobj)
 /* same.						  */
 /*--------------------------------------------------------*/
 
-findsubschems(toppage, cschem)
-  int toppage;
-  objectptr cschem;
+void findsubschems(int toppage, objectptr cschem)
 {
    genericptr *cgen;
 
@@ -380,8 +468,7 @@ findsubschems(toppage, cschem)
 /* Copy label to corresponding schematic/symbol		  */
 /*--------------------------------------------------------*/
 
-copypinlabel(pinlab)
-  labelptr pinlab;
+void copypinlabel(labelptr pinlab)
 {
    labelptr *newlabel, tlabel;
    genericptr *tgen;
@@ -395,7 +482,7 @@ copypinlabel(pinlab)
 		tgen++) {
          if ((*tgen)->type == LABEL) {
 	    tlabel = TOLABEL(tgen);
-	    if (!strcmp(tlabel->string, pinlab->string)) return;
+	    if (!stringcomp(tlabel->string, pinlab->string)) return;
          }
       }
 
@@ -405,9 +492,7 @@ copypinlabel(pinlab)
       (*newlabel)->justify = pinlab->justify;
       (*newlabel)->color = pinlab->color;
       (*newlabel)->scale = pinlab->scale;
-      (*newlabel)->string = (char *)malloc((1 + strlen(pinlab->string))
-		* sizeof(char));
-      noparmstrcpy((*newlabel)->string, pinlab->string);
+      (*newlabel)->string = stringcopy(pinlab->string);
 
       /* place label just outside bounding box, then recompute bbox */
 
@@ -483,8 +568,7 @@ int checksym(objectptr symobj, char *cname)
 /* Find location of corresponding pin in symbol/schematic */
 /*--------------------------------------------------------*/
 
-labelptr findotherpin(curlabel)
-  labelptr curlabel;
+labelptr findotherpin(labelptr curlabel)
 {
    objectptr other = objectdata->symschem;
    genericptr *tgen;
@@ -496,7 +580,7 @@ labelptr findotherpin(curlabel)
    for (tgen = other->plist; tgen < other->plist + other->parts; tgen++) {
       if ((*tgen)->type == LABEL) {
 	 tlab = TOLABEL(tgen);
-	 if (!strcmp(tlab->string, curlabel->string)) return tlab;
+	 if (!stringcomp(tlab->string, curlabel->string)) return tlab;
       }
    }
    return NULL;
@@ -508,7 +592,7 @@ labelptr findotherpin(curlabel)
 /*  there is no corresponding schematic/symbol, nothing happens.	*/
 /*----------------------------------------------------------------------*/
 
-XtCallbackProc swapschem(Widget w, caddr_t mode, caddr_t calldata)
+void swapschem(Widget w, u_int mode, caddr_t calldata)
 {
    objectptr savepage = objectdata;
    labelptr  *pinlab;
@@ -522,7 +606,7 @@ XtCallbackProc swapschem(Widget w, caddr_t mode, caddr_t calldata)
 
    /* Create symbol or schematic, if allowed by mode */
 
-   if ((objectdata->symschem == NULL) && (mode != (caddr_t)0)) {
+   if ((objectdata->symschem == NULL) && (mode != 0)) {
       if (objectdata->schemtype == SYMBOL) {
 	 int tpage;
 
@@ -579,7 +663,7 @@ XtCallbackProc swapschem(Widget w, caddr_t mode, caddr_t calldata)
 			objectdata->plist + objectdata->parts; tgen++) {
 		  if ((*tgen)->type == LABEL) {
 		     tlab = TOLABEL(tgen);
-	             if (!strcmp(tlab->string, lpin->string)) lflag = True;
+	             if (!stringcomp(tlab->string, lpin->string)) lflag = True;
 		  }
       	       }
 	       if (lflag == True) continue;
@@ -593,9 +677,7 @@ XtCallbackProc swapschem(Widget w, caddr_t mode, caddr_t calldata)
 	       (*pinlab)->position.x = 0;
 	       (*pinlab)->position.y = objectdata->parts * (TEXTHEIGHT + 10);
 	       u2u_snap(&((*pinlab)->position));
-	       (*pinlab)->string = (char *)malloc((1 + strlen(lpin->string))
-			* sizeof(char));
-	       strcpy((*pinlab)->string, lpin->string);
+	       (*pinlab)->string = stringcopy(lpin->string);
 	       objectdata->parts++;
 	    }
          }
@@ -641,7 +723,7 @@ void schemdisassoc()
 /* context (associate/disassociate, symbol or schematic)	*/
 /*--------------------------------------------------------------*/
 
-XtCallbackProc startschemassoc(Widget w, caddr_t clientdata, caddr_t calldata)
+void startschemassoc(Widget w, caddr_t clientdata, caddr_t calldata)
 {
    if (objectdata->symschem != NULL)
       schemdisassoc();
@@ -649,12 +731,12 @@ XtCallbackProc startschemassoc(Widget w, caddr_t clientdata, caddr_t calldata)
       eventmode = ASSOC_MODE;
       if (objectdata->schemtype == SCHEMATIC) {
 	 /* Find a symbol to associate */
-	 startcatalog(w, Number(LIBLIB), NULL);
+	 startcatalog(w, LIBLIB, NULL);
 	 Wprintf("Click on library page, then symbol to associate.");
       }
       else {
 	 /* Find a schematic (page) to associate */
-	 startcatalog(w, Number(PAGELIB), NULL);
+	 startcatalog(w, PAGELIB, NULL);
 	 Wprintf("Click on schematic page to associate.");
       }
    }
