@@ -1,6 +1,6 @@
 /*-------------------------------------------------------------------------*/
 /* selection.c --- xcircuit routines handling element selection etc.	   */
-/* Copyright (c) 2000  Tim Edwards, Johns Hopkins University       	   */
+/* Copyright (c) 2001  Tim Edwards, Johns Hopkins University       	   */
 /*-------------------------------------------------------------------------*/
 
 /*-------------------------------------------------------------------------*/
@@ -8,8 +8,11 @@
 /*-------------------------------------------------------------------------*/
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
+#if !defined(__DARWIN__)
 #include <malloc.h>
+#endif
 #include <math.h>
 
 #include <X11/Intrinsic.h>
@@ -20,9 +23,14 @@
 /* Local includes							   */
 /*-------------------------------------------------------------------------*/
 
+#include "xcircuit.h"
 #include "cursors.h"
 #include "colordefs.h"
-#include "xcircuit.h"
+
+/*----------------------------------------------------------------------*/
+/* Function prototype declarations                                      */
+/*----------------------------------------------------------------------*/
+#include "prototypes.h"
 
 /*-------------------------------------------------------------------------*/
 /* Global Variable definitions						   */
@@ -37,24 +45,15 @@ extern Clientdata areastruct;
 extern char _STR[150];
 extern short attachto;
 extern short textpos, textend;
+extern objectpair *pushlist;
 
 #define RADFAC 0.0174532925199   /* pi / 180 */
 
-/*----------------------------------------------------------------------*/
-/* Declarations of functions 						*/
-/*----------------------------------------------------------------------*/
-
-extern long finddist();
-extern TextExtents ULength();
-
-short *selectobject(short, objectptr);
-XtEventHandler trackselarea();
-
 /*----------------------------------------------------------------------*/
 /* Change filter to determine what types can be selected		*/
 /*----------------------------------------------------------------------*/
 
-XtCallbackProc selectfilter(Widget w, caddr_t value, caddr_t calldata)
+void selectfilter(Widget w, caddr_t value, caddr_t calldata)
 {
    short bitwise = (int) value & 0xffff;
    Boolean bval = (areastruct.filter & bitwise) ? True : False;
@@ -71,8 +70,7 @@ XtCallbackProc selectfilter(Widget w, caddr_t value, caddr_t calldata)
 /* Look at select stack to see if there are any selects; call select, if not. */
 /*----------------------------------------------------------------------------*/
 
-Boolean checkselect(value)
-  short value;
+Boolean checkselect(short value)
 {
    short *check;
  
@@ -91,25 +89,25 @@ Boolean checkselect(value)
 /* Draw a selected item */
 /*----------------------*/
 
-void geneasydraw(short instance, short mode, objectptr curobj)
+void geneasydraw(short instance, short mode, objectptr curobj, objinstptr curinst)
 {
    genericptr elementptr = *(curobj->plist + instance);
 
    switch ((*(curobj->plist + instance))->type) {
       case ARC:
-         UDrawArc(areastruct.topobject, (arcptr)elementptr);
+         UDrawArc(curinst, (arcptr)elementptr);
 	 break;
       case POLYGON:
-         UDrawPolygon(areastruct.topobject, (polyptr)elementptr);
+         UDrawPolygon(curinst, (polyptr)elementptr);
 	 break;
       case SPLINE:
-	 UDrawSpline(areastruct.topobject, (splineptr)elementptr);
+	 UDrawSpline(curinst, (splineptr)elementptr);
 	 break;
       case PATH:
-	 UDrawPath(areastruct.topobject, (pathptr)elementptr);
+	 UDrawPath(curinst, (pathptr)elementptr);
 	 break;
       case LABEL:
-         UDrawString(areastruct.topobject, (labelptr)elementptr, mode);
+         UDrawString(curinst, (labelptr)elementptr, mode);
 	 break;
       case OBJECT:
          UDrawObject((objinstptr)elementptr, 
@@ -122,14 +120,14 @@ void geneasydraw(short instance, short mode, objectptr curobj)
 /* Draw a selected item, including selection color */
 /*-------------------------------------------------*/
 
-void gendrawselected(short *newselect, objectptr curobj)
+void gendrawselected(short *newselect, objectptr curobj, objinstptr curinst)
 {
    XSetFunction(dpy, areastruct.gc, GXcopy);
    XSetForeground(dpy, areastruct.gc, BACKGROUND);
-   geneasydraw(*newselect, DOFORALL, curobj);
+   geneasydraw(*newselect, DOFORALL, curobj, curinst);
    XSetFunction(dpy, areastruct.gc, GXxor);
    XSetForeground(dpy, areastruct.gc, SELECTCOLOR ^ BACKGROUND);
-   geneasydraw(*newselect, DOFORALL, curobj);
+   geneasydraw(*newselect, DOFORALL, curobj, curinst);
 }
 
 /*---------------------------------------------------*/
@@ -156,7 +154,7 @@ short *allocselect()
 /* Set Options menu according to 1st selection	   */
 /*-------------------------------------------------*/
 
-setoptionmenu()
+void setoptionmenu()
 {
    short      *mselect;
    labelptr   mlabel;
@@ -186,8 +184,7 @@ setoptionmenu()
 	    return;
 	 case LABEL:
 	    mlabel = SELTOLABEL(mselect);
-	    setfontmarks((unsigned short)(*(mlabel->string + 1)) - FONT_START,
-		mlabel->justify);
+	    setfontmarks(mlabel->string->data.font, mlabel->justify);
 	    return;
       }
    }
@@ -197,7 +194,8 @@ setoptionmenu()
 /* Test of point being inside of a box */
 /*-------------------------------------*/
 
-int test_insideness(int tx, int ty, XPoint *boxpoints) {
+int test_insideness(int tx, int ty, XPoint *boxpoints)
+{
    int i, stval = 0; 
    XPoint *pt1, *pt2;
    int stdir;   
@@ -216,9 +214,7 @@ int test_insideness(int tx, int ty, XPoint *boxpoints) {
 /* Search for selection among path components */
 /*--------------------------------------------*/
 
-Boolean pathselect(curgen, class)
-  genericptr *curgen;
-  short class;
+Boolean pathselect(genericptr *curgen, short class)
 {
    /*----------------------------------------------------------------------*/
    /* wirelim is the distance, in user-space units, at which an element is */
@@ -321,19 +317,16 @@ Boolean pathselect(curgen, class)
 /* Select one of the elements on-screen */
 /*--------------------------------------*/
 
-short *selectobject(short class, objectptr selobj)
+short *selectobject(short class, objectptr selobj, objinstptr selinst)
 {
    short	*newselect, *foundtmp, *foundlist;
 
    genericptr	*curgen;
-
    XPoint 	bboxpts[4], newboxpts[4];
    XEvent 	event;
    XButtonEvent *buttonevent = (XButtonEvent *)(&event);
    Boolean	selected, unselect = False;
-   short	stval, i, j, found = 0;
-   long		newdist;
-   Arg		wargs[2];
+   short	found = 0;
 
    XDefineCursor(dpy, win, QUESTION);
 
@@ -397,7 +390,7 @@ short *selectobject(short class, objectptr selobj)
 
          /* now look among the labels */
 
-	 TextExtents tmpext = ULength(curlab->string, 0.0, 0, NULL);
+	 TextExtents tmpext = ULength(curlab->string, selinst, 0.0, 0, NULL);
 
          bboxpts[0].x = bboxpts[1].x = (curlab->justify & NOTLEFT ?
 	    (curlab->justify & RIGHT ? -tmpext.width : -tmpext.width / 2) : 0);
@@ -435,7 +428,6 @@ short *selectobject(short class, objectptr selobj)
          /* Check among the paths */
 
 	 genericptr *pathp;
-	 Boolean lselect;
 
 	 /* Accept path if any subcomponent of the path is accepted */
 	 /* Record which part was selected in the "subeditpart" variable */
@@ -487,7 +479,7 @@ short *selectobject(short class, objectptr selobj)
 
          XSetFunction(dpy, areastruct.gc, GXcopy);
          XTopSetForeground(GSELTOCOLOR(selobj, newselect));
-         geneasydraw(*newselect, DEFAULTCOLOR, selobj);
+         geneasydraw(*newselect, DEFAULTCOLOR, selobj, selinst);
          areastruct.selects--;
          for (desel = newselect; desel < areastruct.selectlist +
 	        areastruct.selects; desel++)
@@ -509,14 +501,14 @@ short *selectobject(short class, objectptr selobj)
 	 newselect = allocselect();
 	 *newselect = *foundlist;
 	 free(foundlist);
-	 gendrawselected(newselect, selobj);
+	 gendrawselected(newselect, selobj, selinst);
 	 break;
 
       default: {
 	 XSetFunction(dpy, areastruct.gc, GXcopy);
 	 for (foundtmp = foundlist; foundtmp < foundlist + found; foundtmp++) {
             XSetForeground(dpy, areastruct.gc, QUERYCOLOR);
-            geneasydraw(*foundtmp, DOFORALL, selobj);
+            geneasydraw(*foundtmp, DOFORALL, selobj, selinst);
 	    sprintf(_STR, "Click to accept/reject: %d of %d",
 		(int)(foundtmp - foundlist + 1), found);
 	    Wprintf(_STR);
@@ -530,13 +522,13 @@ short *selectobject(short class, objectptr selobj)
 	    }
 	    if (buttonevent->button == Button3) {
                XTopSetForeground(GSELTOCOLOR(selobj, foundtmp));
-               geneasydraw(*foundtmp, DEFAULTCOLOR, selobj);
+               geneasydraw(*foundtmp, DEFAULTCOLOR, selobj, selinst);
 	    }
 	    else {
 	       newselect = allocselect();
                *newselect = *foundtmp;
                XSetForeground(dpy, areastruct.gc, SELECTCOLOR);
-               geneasydraw(*newselect, DOFORALL, selobj);
+               geneasydraw(*newselect, DOFORALL, selobj, selinst);
 	    }
          }
       }  break;
@@ -569,12 +561,12 @@ short *selectobject(short class, objectptr selobj)
       XSetForeground(dpy, areastruct.gc, BACKGROUND);
       for (nselect = areastruct.selectlist; nselect < areastruct.selectlist
 	   + areastruct.selects; nselect++)
-         geneasydraw(*nselect, DOFORALL, selobj);
+         geneasydraw(*nselect, DOFORALL, selobj, selinst);
       XSetFunction(dpy, areastruct.gc, GXxor);
       XSetForeground(dpy, areastruct.gc, SELECTCOLOR ^ BACKGROUND);
       for (nselect = areastruct.selectlist; nselect < areastruct.selectlist
 	   + areastruct.selects; nselect++)
-         geneasydraw(*nselect, DOFORALL, selobj);
+         geneasydraw(*nselect, DOFORALL, selobj, selinst);
    }
 
    setoptionmenu();
@@ -594,8 +586,7 @@ short *selectobject(short class, objectptr selobj)
 /* select arc, curve, and polygon objects from a defined box area */
 /*----------------------------------------------------------------*/
 
-Boolean areaelement(curgen)
-  genericptr *curgen;
+Boolean areaelement(genericptr *curgen)
 {
    Boolean selected;
    pointlist    currentpt;
@@ -651,6 +642,8 @@ void selectarea()
    short	*newselect;
    genericptr   *curgen, *pathgen;
    Boolean	selected;
+   stringpart	*strptr;
+   int		locpos;
 
    /* put points of area bounding box into proper order */
 
@@ -709,7 +702,7 @@ void selectarea()
 			&(newboxpts[j].y), -1);
 	   }
 #endif
-	   tmpext = ULength(slab->string, 0.0, 0, NULL);
+	   tmpext = ULength(slab->string, NORMINST, 0.0, 0, NULL);
 	   adj.x = (slab->justify & NOTLEFT ? (slab->justify & RIGHT ? 
 			tmpext.width : tmpext.width >> 1) : 0);
 	   adj.y = (slab->justify & NOTBOTTOM ? (slab->justify & TOP ? 
@@ -720,12 +713,12 @@ void selectarea()
 	   /* do an insideness test with the select box.		*/
 
 	   state = tmpl2 = 0;
-	   for (j = 2; j < strlen(slab->string); j++) {
-	      if (*(slab->string + j) == TEXT_ESC
-			&& *(slab->string + j + 1) == RETURN)
-		 tmpl2 = 0;
+	   for (j = 0; j < stringlength(slab->string, True, NORMINST); j++) {
+	      strptr = findstringpart(j, &locpos, slab->string, NORMINST);
+	      if (locpos < 0) continue;	  /* only look at printable characters */
+	      if (strptr->type == RETURN) tmpl2 = 0;
 	      tmpl1 = tmpl2;
-	      tmpext = ULength(slab->string, 0.0, j + 1, NULL);
+	      tmpext = ULength(slab->string, NORMINST, 0.0, j + 1, NULL);
 	      tmpl2 = tmpext.width;
 	      isect = test_insideness(((tmpl1 + tmpl2) >> 1) - adj.x,
 			(tmpext.base + (BASELINE >> 1)) - adj.y, newboxpts);
@@ -736,7 +729,7 @@ void selectarea()
 		    state = 1;
 		    selected = True;
 		    textend = j;
-		    if ((textend > 1) && (*(slab->string + textend - 1) == TEXT_ESC))
+		    if ((textend > 1) && strptr->type != TEXT_STRING)
 		       textend--;
 		 }
 	      }
@@ -798,9 +791,7 @@ void selectarea()
 /* start deselection mode */
 /*------------------------*/
 
-void startdesel(w, clientdata, nulldata)
-  Widget w;
-  caddr_t clientdata, nulldata;
+void startdesel(Widget w, caddr_t clientdata, caddr_t calldata)
 {
    if (eventmode == NORMAL_MODE) {
       eventmode = DESEL_MODE;
@@ -808,25 +799,11 @@ void startdesel(w, clientdata, nulldata)
    }
 }
 
-/*------------------------------------------*/
-/* deselect all objects on the select stack */
-/*------------------------------------------*/
+/*---------------------------------------------------------*/
+/* Redraw all the selected objects in their normal colors. */
+/*---------------------------------------------------------*/
 
-void deselect(w, clientdata, calldata)
-  Widget w;
-  caddr_t clientdata;
-  caddr_t calldata;
-{
-   objectselect(-ANY);
-
-   /* return Option menu to default state */
-
-   setoptionmenu();
-}
-
-/*------------------------------------------*/
-
-void objectdeselect()
+void drawselects(objectptr thisobj, objinstptr thisinst)
 {
    short *lastselect;
 
@@ -839,8 +816,18 @@ void objectdeselect()
    for (lastselect = areastruct.selectlist; lastselect < areastruct.selectlist 
 	+ areastruct.selects; lastselect++) {
       XTopSetForeground(SELTOCOLOR(lastselect)); 
-      easydraw(*lastselect, SELTOCOLOR(lastselect));
+      geneasydraw(*lastselect, SELTOCOLOR(lastselect), thisobj, thisinst);
    }
+}
+
+/*--------------------------------------------------------------*/
+/* Free memory from the selection list and set menu/toolbar	*/
+/* items back to default values.				*/
+/*--------------------------------------------------------------*/
+
+void clearselects()
+{
+   if (areastruct.selects == 0) return;
 
    areastruct.selects = 0;
    free(areastruct.selectlist);
@@ -850,67 +837,91 @@ void objectdeselect()
    setdefaultfontmarks();
 }
 
-/*------------------------------------------------------------------------*/
-/* Select the nearest polygon, searching down the hierarchy if necessary. */
-/* Return pointers to the polygon and the object containing the polygon.  */
-/*------------------------------------------------------------------------*/
+/*--------------------------------------------------------------*/
+/* Unselect all the selected objects and free memory from the	*/
+/* selection list.						*/
+/*--------------------------------------------------------------*/
 
-objectptr recurselect(polyptr *polyreturn, objectptr selobj)
+void objectdeselect()
 {
+   drawselects(objectdata, NORMINST);
+   clearselects();
+}
+
+/*----------------------------------------------------------------------*/
+/* Select the nearest polygon, searching the hierarchy if necessary.	*/
+/* Return an objectpair pointer to a linked list containing the 	*/
+/* hierarchy of objects, with the topmost objectpair also containing a	*/
+/* pointer to the polygon found.					*/
+/* Allocates memory for the returned linked list which must be freed by	*/
+/* the calling routine.							*/
+/*----------------------------------------------------------------------*/
+
+genericptr recurselect(objectpair **seltop)
+{
+   genericptr rgen = NULL;
    short *selectobj;
+   objectptr selobj;
    objinstptr selinst;
-   objectptr selreturn;
    XPoint savesave, tmppt;
 
-   if (areastruct.selects > 0) {
-      areastruct.selects = 0;
-      free(areastruct.selectlist);
+   if (*seltop == NULL) {
+      fprintf(stderr, "Error: recurselect called with NULL objectpair pointer\n");
+      return NULL;
    }
 
-   if ((selectobj = selectobject(POLYGON, selobj)) != NULL) {
-      *polyreturn = TOPOLY(selobj->plist + (*selectobj));
-      areastruct.selects = 0;
-      free(areastruct.selectlist);
-      /* printf("polygon found in object %s\n", selobj->name); */
-      selreturn = selobj;
+   selobj = (*seltop)->thisobject;
+   selinst = (*seltop)->thisinst;
+
+   if ((selectobj = selectobject(POLYGON | LABEL, selobj, selinst)) != NULL) {
+      rgen = *(selobj->plist + (*selectobj));
+      /* printf("%s found in object %s\n", (rgen->type == LABEL) ? "label" : "polygon",
+			selobj->name); */
+      drawselects(selobj, selinst);
+      clearselects();
    }
-   else if ((selectobj = selectobject(OBJECT, selobj)) != NULL) {
-      if ((*(selobj->plist + *selectobj))->type == OBJECT) {
-         selinst = TOOBJINST(selobj->plist + (*selectobj));	
-         /* Translate areastruct.save into object's own coordinate system */
-         savesave.x = areastruct.save.x;
-         savesave.y = areastruct.save.y;
-         InvTransformPoints(&areastruct.save, &tmppt, 1, selinst->position,
+   else if ((selectobj = selectobject(OBJECT, selobj, selinst)) != NULL) {
+      objinstptr selinst = TOOBJINST(selobj->plist + (*selectobj));	
+      objectpair *selnew = (objectpair *)malloc(sizeof(objectpair));
+      drawselects(selobj, selinst);
+      clearselects();
+
+      /* Link hierarchy information to the objectpair linked list */
+      selnew->thisinst = selinst;
+      selnew->thisobject = selinst->thisobject;
+      selnew->nextpair = NULL;
+      (*seltop)->nextpair = selnew;
+	 
+      /* Translate areastruct.save into object's own coordinate system */
+      savesave.x = areastruct.save.x;
+      savesave.y = areastruct.save.y;
+      InvTransformPoints(&areastruct.save, &tmppt, 1, selinst->position,
 		selinst->scale, selinst->rotation);
-         areastruct.save.x = tmppt.x;
-         areastruct.save.y = tmppt.y;
-         /* printf("objinst %s found in object %s; searching recursively\n",
+      areastruct.save.x = tmppt.x;
+      areastruct.save.y = tmppt.y;
+      /* printf("objinst %s found in object %s; searching recursively\n",
 		selinst->thisobject->name, selobj->name); */
-         /* printf("cursor position originally (%d, %d); in new object is (%d, %d)\n",
+      /* printf("cursor position originally (%d, %d); in new object is (%d, %d)\n",
 		savesave.x, savesave.y,
 		areastruct.save.x, areastruct.save.y); */
-	 UPushCTM();
-	 UPreMultCTM(DCTM, selinst->position, selinst->scale, selinst->rotation);
-         selreturn = recurselect(polyreturn, selinst->thisobject);
-         UPopCTM();
-         areastruct.save.x = savesave.x;
-         areastruct.save.y = savesave.y;
-      }
+      UPushCTM();
+      UPreMultCTM(DCTM, selinst->position, selinst->scale, selinst->rotation);
+      rgen = recurselect(&selnew);
+      UPopCTM();
+      areastruct.save.x = savesave.x;
+      areastruct.save.y = savesave.y;
    }
-   else {
-      /* printf("nothing found in object %s\n", selobj->name); */
-      selreturn = (objectptr)NULL;
-   }
-   return selreturn;
+   /* else
+      printf("nothing found in object %s\n", selobj->name); */
+
+   return rgen;
 }
 
 /*----------------------------------*/
 /* Start drawing a select area box. */
 /*----------------------------------*/
 
-XtTimerCallbackProc startselect(id, clientdata)
-  XtIntervalId  id;
-  caddr_t       clientdata;
+void startselect(XtIntervalId id, caddr_t clientdata)
 {
    if (eventmode == NORMAL_MODE) eventmode = SELAREA_MODE;
    areastruct.origin.x = areastruct.save.x;
@@ -924,15 +935,12 @@ XtTimerCallbackProc startselect(id, clientdata)
 /* Track a select area box */
 /*-------------------------*/
 
-XtEventHandler trackselarea(w, clientdata, event)
-  Widget w;
-  XtPointer clientdata;
-  XMotionEvent *event;
+void trackselarea(Widget w, XtPointer clientdata, XMotionEvent *event)
 {
    Window nullwin;
    int    nullint, xpos, ypos;
-   XPoint       newpos;
-   unsigned int   nullui;
+   XPoint newpos;
+   u_int  nullui;
 
    XQueryPointer(dpy, win, &nullwin, &nullwin, &nullint, &nullint, &xpos, &ypos,
 	&nullui);
