@@ -1,6 +1,6 @@
 /*-------------------------------------------------------------------------*/
 /* xcircuit.c --- An X-windows program for drawing circuit diagrams	   */
-/* Copyright (c) 2000  Tim Edwards, Johns Hopkins University        	   */
+/* Copyright (c) 2001  Tim Edwards, Johns Hopkins University        	   */
 /*-------------------------------------------------------------------------*/
 
 /*-------------------------------------------------------------------------*/
@@ -11,7 +11,9 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#if !defined(__DARWIN__)
 #include <malloc.h>
+#endif
 #include <signal.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -39,10 +41,6 @@
 #include "Xw/TextEdit.h"
 #include "Xw/Toggle.h"
 
-#ifdef HAVE_XPM
-#include <X11/xpm.h>
-#endif
-
 #ifndef P_tmpdir
 #define P_tmpdir TMPDIR
 #endif
@@ -51,12 +49,16 @@
 /* Local includes							   */
 /*-------------------------------------------------------------------------*/
 
+#include "xcircuit.h"
 #include "cursors.h"
 #include "colordefs.h"
-#include "xcircuit.h"
-#include "menudefs.h"
 #include "menudep.h"
 
+/*----------------------------------------------------------------------*/
+/* Function prototype declarations                                      */
+/*----------------------------------------------------------------------*/
+#include "prototypes.h"
+
 /*-------------------------------------------------------------------------*/
 /* Local defines							   */
 /*-------------------------------------------------------------------------*/
@@ -122,13 +124,14 @@ XtIntervalId printtime_id;
 short beeper;
 short fontcount;
 fontinfo *fonts;
-Boolean shiftstate;
 
-extern short eventmode, textpos, pushes, help_up;
+extern short eventmode, textpos, help_up;
 extern float version;
 extern menustruct TopButtons[];
 extern toolbarstruct ToolBar[];
 extern short maxbuttons, toolbuttons;
+extern Pixmap helppix;
+extern objectpair *pushlist;
 
 #ifdef DOUBLEBUFFER
 extern Pixmap dbuf;
@@ -144,33 +147,6 @@ XContext XUniqueContext()
 #endif
 /* End of bad hack. . . */
 
-/*------------------------------------------------------------------------*/
-/* Declarations of functions defined externally to xcircuit.c		  */
-/*------------------------------------------------------------------------*/
-
-extern void selectbutton(), releasebutton(), keyhandler(), keyrelease(),
-	Wprintf(), setfloat(), setscalex(), setscaley(),
-	calcbboxvalues(), setfilename(),
-	setpagelabel(), genfilelist(), newfilelist();
-extern void redrawtext(), setpmode(), setorient(), getgeneric(), setdscale(),
-	W2printf(), initsplines(), setpagesize();
-extern void startloadfile(), autoscale();
-extern float getpsscale(float, short);
-extern int xc_alloccolor(), lookdirectory(), exit_gs();
-extern short execcommand();
-extern XtCallbackProc resizearea(), drawarea(), setfile();
-extern XtCallbackProc drawhbar(), drawvbar();
-extern XtCallbackProc autoset(), autostop();
-extern XtTimerCallbackProc savetemp();
-
-void renamepage(), W1printf();
-void makecursors();
-
-XtEventHandler panhbar(), panvbar();
-XtCallbackProc endhbar(), endvbar();
-XtCallbackProc setcolor(), updatename(), updatetext(), DoNothing();
-XtCallbackProc resizetoolbar();
-
 /*-------------------------------------------------------------------------*/
 /* Initial Resource Management						   */
 /*-------------------------------------------------------------------------*/
@@ -256,9 +232,7 @@ static XtResource resources[] = {
 /* Update the list of colors in the colormap				   */
 /*-------------------------------------------------------------------------*/
 
-addtocolorlist(button, cvalue)
-  Widget button;
-  int    cvalue;
+void addtocolorlist(Widget button, int cvalue)
 {
    number_colors++;
    colorlist = (colorindex *)realloc(colorlist, number_colors *
@@ -271,12 +245,12 @@ addtocolorlist(button, cvalue)
    XQueryColor(dpy, cmap, &(colorlist[number_colors - 1].color));
 }
 
-/*-------------------------------------------------------------------------*/
-/* Add a new color button to the color menu				   */
-/*-------------------------------------------------------------------------*/
+/*----------------------------------------------------------------------*/
+/* Add a new color button to the color menu				*/
+/* 	called if new color button needs to be made 			*/
+/*----------------------------------------------------------------------*/
 
-void addnewcolorentry(ccolor)	/* called if new color button needs to be made */
-   int ccolor;
+void addnewcolorentry(int ccolor)
 {
    Widget colormenu, newbutton;
    Arg wargs[2];
@@ -310,11 +284,8 @@ void addnewcolorentry(ccolor)	/* called if new color button needs to be made */
 /*   menu structure is > 0.						*/
 /*----------------------------------------------------------------------*/
 
-void makesubmenu(menuname, attachname, buttonmenu, arraysize, manager)
-char *menuname, *attachname;
-menuptr buttonmenu;
-int arraysize;
-Widget manager;
+void makesubmenu(char *menuname, char *attachname, menuptr buttonmenu,
+		int arraysize, Widget manager)
 {
    short i, n = 0;
    int cval;
@@ -444,15 +415,14 @@ void createtoolbar (Widget abform, Widget aform)
 /* Toolbar Resize (This is not working right).				*/
 /*----------------------------------------------------------------------*/
 
-XtCallbackProc resizetoolbar()
+void resizetoolbar()
 {
-   int i, j, n = 0, bcol = 0; 
+   int i, n = 0, bcol = 0; 
    int max_width = 0, max_height = 0, tot_width = 0;
    Arg	wargs[4];
    Widget bwptr, lbwptr, bmax, lbmax;
-   Dimension t_height, t_width, bheight, bwidth;
+   Dimension t_height, bheight, bwidth;
    Boolean is_top = True;
-   XEvent discard;
 
    if (!XtIsRealized(toolbar)) return;
 
@@ -582,14 +552,19 @@ void dointr(int signum)
 /* Quit xcircuit */
 /*---------------*/
 
-XtCallbackProc quit(w, clientdata, calldata)
-  Widget w;
-  caddr_t clientdata, calldata;
+void quit(Widget w, caddr_t clientdata, caddr_t calldata)
 {
    int i;
+   Matrixptr curmatrix, dmatrix;
 
    /* free up CTM Matrix Stack */
-   free(areastruct.MatStack);
+   curmatrix = areastruct.MatStack;
+   while (curmatrix != NULL) {
+      dmatrix = curmatrix->nextmatrix;
+      free(curmatrix);
+      curmatrix = dmatrix;
+   }
+   areastruct.MatStack = NULL;
 
    /* free the colormap if a new one has been installed */
 
@@ -601,6 +576,11 @@ XtCallbackProc quit(w, clientdata, calldata)
 
    exit_gs();
 
+#ifdef HAVE_PYTHON
+   /* exit the Python interpreter, if enabled */
+   exit_interpreter();
+#endif
+
    /* remove any temporary files created for background rendering */
 
    for (i = 0; i < xobjs.pages; i++) {
@@ -632,9 +612,7 @@ XtCallbackProc quit(w, clientdata, calldata)
 /* Null routine */
 /*--------------*/
 
-XtCallbackProc DoNothing(w, clientdata, calldata)
-  Widget w;
-  caddr_t clientdata, calldata;
+void DoNothing(Widget w, caddr_t clientdata, caddr_t calldata)
 {
    /* Nothing here! */
 }
@@ -645,9 +623,7 @@ XtCallbackProc DoNothing(w, clientdata, calldata)
 /* Propogate any key event from the dialog box into the textedit widget    */
 /*-------------------------------------------------------------------------*/
 
-XtEventHandler propevent(w, editwidget, event)
-  Widget w, editwidget;
-  XEvent *event;
+void propevent(Widget w, Widget editwidget, XEvent *event)
 {
    Window ewin = XtWindow(editwidget);
 
@@ -659,14 +635,15 @@ XtEventHandler propevent(w, editwidget, event)
 /* Destroy an interactive text-editing popup box 			   */
 /*-------------------------------------------------------------------------*/
 
-XtCallbackProc destroypopup(button, callstruct, calldata)
-  Widget button;
-  popupstruct *callstruct;
-  caddr_t calldata;
+void destroypopup(Widget button, popupstruct *callstruct, caddr_t calldata)
 {
    Arg	wargs[1];
 
-   if(XtNameToWidget(callstruct->popup, "help2") != NULL) help_up = False;
+   if(XtNameToWidget(callstruct->popup, "help2") != NULL) {
+      help_up = False;
+      XFreePixmap(dpy, helppix);
+      helppix = (Pixmap)NULL;
+   }
 
    if (callstruct->buttonptr->button != NULL) {  
 
@@ -693,13 +670,8 @@ XtCallbackProc destroypopup(button, callstruct, calldata)
 /* Pull text from the popup prompt buffer into a global string variable    */
 /*-------------------------------------------------------------------------*/
 
-XtCallbackProc xcgettext(button, callstruct, calldata)
-  Widget 	button;
-  popupstruct	*callstruct;
-  caddr_t 	calldata;
+void xcgettext(Widget button, popupstruct *callstruct, caddr_t calldata)
 {
-   void *function = callstruct->setvalue;
-
    sprintf(_STR2, "%.249s", XwTextCopyBuffer(callstruct->textw)); 
 
    /* functions which use the file selector should look for directory name */
@@ -713,9 +685,9 @@ XtCallbackProc xcgettext(button, callstruct, calldata)
       }
    }
 
-   /* Pop down the widget now (for functions like execscript() which	*/
-   /* may want to interactively control the window contents), but do	*/
-   /* not destroy it until the function has returned.			*/
+   /* Pop down the widget now (for functions like execscript() which    */
+   /* may want to interactively control the window contents), but do    */
+   /* not destroy it until the function has returned.                   */
 
    XtPopdown(callstruct->popup);
 
@@ -734,10 +706,7 @@ XtCallbackProc xcgettext(button, callstruct, calldata)
 /* Grab text from the "output properties" window			   */
 /*-------------------------------------------------------------------------*/
 
-XtCallbackProc getproptext(button, callstruct, calldata)
-  Widget 	button;
-  propstruct	*callstruct;
-  caddr_t 	calldata;
+void getproptext(Widget button, propstruct *callstruct, caddr_t calldata)
 {
    /* xobjs.pagelist[areastruct.page]->filename can be realloc'd by the */
    /* call to *(callstruct->setvalue), so callstruct->dataptr may no    */
@@ -798,10 +767,7 @@ XtCallbackProc getproptext(button, callstruct, calldata)
 /* Update scale, width, and height in response to change of one of them	   */
 /*-------------------------------------------------------------------------*/
 
-XtCallbackProc updatetext(button, callstruct, calldata)
-  Widget	button;
-  WidgetList	callstruct;
-  caddr_t	calldata;
+void updatetext(Widget button, WidgetList callstruct, caddr_t calldata)
 {
    float oscale, psscale;
    char edit[3][50];
@@ -833,12 +799,9 @@ XtCallbackProc updatetext(button, callstruct, calldata)
 /* Update the object name in response to a change in filename		   */
 /*-------------------------------------------------------------------------*/
 
-XtCallbackProc updatename(button, callstruct, calldata)
-  Widget        button;
-  WidgetList    callstruct;
-  caddr_t       calldata;
+void updatename(Widget button, WidgetList callstruct, caddr_t calldata)
 {
-   short i, n, posit;
+   short n, posit;
    char  *rootptr;
    Arg   wargs[2]; 
       
@@ -866,22 +829,17 @@ XtCallbackProc updatename(button, callstruct, calldata)
 /* and text and label fields.						   */
 /*-------------------------------------------------------------------------*/
 
-void popupprompt(button, request, current, function, datastruct, fileprompt)
-Widget  button;
-char *request, *current;
-void (*function)();
-buttonsave *datastruct;
-Boolean fileprompt;
+void popupprompt(Widget button, char *request, char *current, void (*function)(),
+	buttonsave *datastruct, Boolean fileprompt)
 {
     Arg         wargs[8];
-    Widget      popup, dialog, okbutton, cancelbutton, entertext, tmpwidget;
+    Widget      popup, dialog, okbutton, cancelbutton, entertext;
     Widget	staticarea;
     XWMHints	*wmhints;	/* for proper input focus */
     Position    xpos, ypos;
     short	n = 0;
     Dimension	height, width, areawidth, areaheight, bwidth, owidth;
     static char defaultTranslations[] = "<Key>Return:	execute()";
-    XGCValues	promptvalues;
     popupstruct	*okaystruct;
 
     height = (current == NULL) ? ROWHEIGHT * 4 : ROWHEIGHT * 5;
@@ -995,33 +953,30 @@ Boolean fileprompt;
 #define MAXPROPS 7
 propstruct okstruct[MAXPROPS], fpokstruct;
 
-XtCallbackProc dooutput(button, clientdata, calldata)
-  Widget        button;
-  caddr_t       clientdata, calldata;
+void dooutput(Widget button, caddr_t clientdata, caddr_t calldata)
 {
    buttonsave  *savebutton;
    Arg         wargs[8];
-   Widget      popup, dialog, okbutton, tmpwidget, titlearea, wrbutton;
+   Widget      popup, dialog, okbutton, titlearea, wrbutton;
    Widget      fpentertext, fpokay, autobutton;
    WidgetList  staticarea, entertext, okays;
    XWMHints    *wmhints;	/* for proper input focus */
    Position    xpos, ypos;
    short       n = 0;
    Dimension   height, width, areawidth, areaheight, bwidth, owidth, wwidth;
-   XGCValues   promptvalues;
    char	       *pdptr;
    short       posit, i;
    popupstruct *donestruct;
-   void		  (*function[MAXPROPS])();
-   XtCallbackProc (*update[MAXPROPS])();
+   void		(*function[MAXPROPS])();
+   void		(*update[MAXPROPS])();
    char	statics[MAXPROPS][50], edit[MAXPROPS][75], request[150];
    char fpedit[75], outname[75];
-   void	*data[MAXPROPS], *objpointer;
+   void	*data[MAXPROPS];
    float oscale, psscale;
    struct stat statbuf;
    static char defaultTranslations[] = "<Key>Return:	execute()";
 
-   if (pushes != 0) {
+   if (pushlist != NULL) {
       Wprintf("Can only save a top-level page!");
       return;
    }
@@ -1317,22 +1272,26 @@ XtCallbackProc dooutput(button, clientdata, calldata)
 /* For formatted strings, format first into _STR   */
 /*-------------------------------------------------*/
 
-XtTimerCallbackProc clrmessage(id, clientdata)
-  XtIntervalId	id;
-  caddr_t	clientdata;
+void clrmessage(XtIntervalId id, caddr_t clientdata)
 {
    char buf1[50], buf2[50];
 
-   measurestr(xobjs.pagelist[areastruct.page]->gridspace, buf1);
-   measurestr(xobjs.pagelist[areastruct.page]->snapspace, buf2);
-   sprintf(_STR, "Grid %.50s : Snap %.50s", buf1, buf2);
-   Wprintf(_STR);
+   /* Don't write over the report of the edit string contents,	*/
+   /* if we're in one of the label edit modes 			*/
+
+   if (eventmode == TEXT2_MODE || eventmode == TEXT3_MODE)
+      charreport(TOLABEL(EDITPART));
+   else {
+      measurestr(xobjs.pagelist[areastruct.page]->gridspace, buf1);
+      measurestr(xobjs.pagelist[areastruct.page]->snapspace, buf2);
+      sprintf(_STR, "Grid %.50s : Snap %.50s", buf1, buf2);
+      Wprintf(_STR);
+   }
 }
 
 /*------------------------------------------------------------------------------*/
 
-void W1printf(string)
-char *string;
+void W1printf(char *string)
 {
    Arg	wargs[1];
 
@@ -1344,8 +1303,7 @@ char *string;
 
 /*------------------------------------------------------------------------------*/
 
-void W2printf(string)
-char *string;
+void W2printf(char *string)
 {
    Arg	wargs[1];
 
@@ -1357,8 +1315,7 @@ char *string;
 
 /*------------------------------------------------------------------------------*/
 
-void Wprintf(string)
-char *string;
+void Wprintf(char *string)
 {
    Arg	wargs[1];
 
@@ -1377,10 +1334,14 @@ char *string;
 
 /*------------------------------------------------------------------------------*/
 
-XtCallbackProc getcommand(Widget cmdw, caddr_t clientdata, caddr_t calldata)
+void getcommand(Widget cmdw, caddr_t clientdata, caddr_t calldata)
 {
    sprintf(_STR2, "%.249s", XwTextCopyBuffer(cmdw));
-   execcommand(0, _STR2);
+#ifdef HAVE_PYTHON
+   execcommand(0, _STR2 + 4);
+#else
+   execcommand(0, _STR2 + 2);
+#endif
    XtRemoveEventHandler(areastruct.area, KeyPressMask, False,
 	(XtEventHandler)propevent, cmdw);
    XtAddCallback(areastruct.area, XtNkeyDown, (XtCallbackProc)keyhandler, NULL);
@@ -1419,9 +1380,15 @@ void docommand()
    }
    else {
       XtManageChild(overlay);
-      XwTextClearBuffer(overlay);
    }
 
+   XwTextClearBuffer(overlay);
+#ifdef HAVE_PYTHON
+   XwTextInsert(overlay, ">>> ");
+#else
+   XwTextInsert(overlay, "? ");
+#endif
+
    /* temporarily redirect all text into the overlay widget */
 
    XtRemoveCallback(areastruct.area, XtNkeyDown, (XtCallbackProc)keyhandler, NULL);
@@ -1435,7 +1402,6 @@ void docommand()
 
 int installowncmap()
 {
-   int i;
    Colormap newcmap;
 
    printf("Installing my own colormap\n");
@@ -1466,15 +1432,14 @@ int installowncmap()
 /* Find the nearest color in the colormap to cvexact and return its pixel value */
 /*------------------------------------------------------------------------------*/
 
-int findnearcolor(cvexact)
-   XColor *cvexact;
+int findnearcolor(XColor *cvexact)
 {
    /* find the nearest-matching color in the colormap */
  
    int i, ncolors = DisplayCells(dpy, DefaultScreen(dpy));
    XColor *cmcolors;
    long rdist, bdist, gdist;
-   unsigned long dist, mindist;
+   u_long dist, mindist;
    int minidx;
 
    cmcolors = (XColor *)malloc(ncolors * sizeof(XColor));
@@ -1524,9 +1489,8 @@ int findnearcolor(cvexact)
 /* converter until after XtInitialize().				   */
 /*-------------------------------------------------------------------------*/
 
-XtConverter CvtStringToPixel(args, nargs, fromVal, toVal)
-   XrmValuePtr args, fromVal, toVal;
-   int	*nargs;
+XtConverter CvtStringToPixel(XrmValuePtr args, int *nargs, XrmValuePtr fromVal,
+	XrmValuePtr toVal)
 {
    static XColor cvcolor;
    XColor cvexact;
@@ -1546,7 +1510,7 @@ XtConverter CvtStringToPixel(args, nargs, fromVal, toVal)
          cvcolor.pixel = findnearcolor(&cvexact);
    }
 
-   toVal->size = sizeof(unsigned long);
+   toVal->size = sizeof(u_long);
    toVal->addr = (caddr_t) &(cvcolor.pixel);
 }
 
@@ -1554,17 +1518,16 @@ XtConverter CvtStringToPixel(args, nargs, fromVal, toVal)
 /* Allocate new color using the CvtStringToPixel routine		   */
 /*-------------------------------------------------------------------------*/
 
-int xc_alloccolor(name)
-  char *name;
+int xc_alloccolor(char *name)
 {
    XrmValue fromC, toC;
-   int zval = 0, i, pixval;
+   int zval = 0, pixval;
 
    fromC.size = strlen(name);
    fromC.addr = name;
 
    CvtStringToPixel(NULL, &zval, &fromC, &toC);
-   pixval = (int)(*((unsigned long *)toC.addr));
+   pixval = (int)(*((u_long *)toC.addr));
 
    return pixval;
 }
@@ -1573,8 +1536,7 @@ int xc_alloccolor(name)
 /* Allocate new color from RGB values (e.g., from PS file "scb" command)   */
 /*-------------------------------------------------------------------------*/
 
-int rgb_alloccolor(red, green, blue)
-  int red, green, blue;
+int rgb_alloccolor(int red, int green, int blue)
 {
    XColor newcolor;
    int i, pixval = -1;
@@ -1657,10 +1619,7 @@ void makecursors()
 
 #ifdef INPUT_FOCUS
 
-void mappinghandler(w, clientdata, event)
-  Widget w;
-  caddr_t clientdata;
-  XEvent *event;
+void mappinghandler(Widget w, caddr_t clientdata, XEvent *event)
 {
    switch(event->type) {
       case MapNotify:
@@ -1677,10 +1636,7 @@ void mappinghandler(w, clientdata, event)
 
 /*----------------------------------------------------------------------*/
 
-void clientmessagehandler(w, clientdata, event)
-  Widget w;
-  caddr_t clientdata;
-  XEvent *event;
+void clientmessagehandler(Widget w, caddr_t clientdata, XEvent *event)
 {
    if (areastruct.mapped == True) {
       /* fprintf(stderr, "Forcing input focus\n"); */
@@ -1694,10 +1650,7 @@ void clientmessagehandler(w, clientdata, event)
 /* Event handler for WM_DELETE_WINDOW message.                          */
 /*----------------------------------------------------------------------*/
 
-void delwin(w, clientdata, event)
-  Widget w;
-  caddr_t clientdata;
-  XClientMessageEvent *event;
+void delwin(Widget w, caddr_t clientdata, XClientMessageEvent *event)
 {
    if (event->type != ClientMessage)
       return;
@@ -1710,21 +1663,18 @@ void delwin(w, clientdata, event)
 /* Main program 							   */
 /*-------------------------------------------------------------------------*/
 
-main(argc, argv) 
-  int argc; 
-  char *argv[]; 
+int main(int argc, char *argv[]) 
 { 
 
 #ifdef HAVE_XPM
    Widget abform;
 #endif
    Widget form, aform, firstbutton, lastbutton, corner;
-   XColor color, ignore;
    XGCValues    values;
    XWMHints	*wmhints;	/* for proper input focus */
    Arg		wargs[12];
-   Pixmap	icon, mask;
-   short i, j, k = 0, n = 0, maxcolors, page;
+   Pixmap	icon;
+   short i, k = 0, n = 0, page;
 
    char		*argv0;		/* find root of argv[0] */
    short initargc = argc;	/* because XtInitialize() absorbs the     */
@@ -1747,12 +1697,13 @@ main(argc, argv)
    /* setlocale() to work.					  */
    /*-------------------------------------------------------------*/
 
-#ifdef linux
-   unsetenv("LC_ALL");
-#endif
 #ifdef SVR4
+   putenv("LC_ALL=en_US");
+   putenv("LC_NUMERIC=en_US");
    putenv("LANG=POSIX");
 #else
+   unsetenv("LC_ALL");
+   unsetenv("LC_NUMERIC");
    setenv("LANG", "POSIX", 1);
 #endif
    setlocale(LC_ALL, "en_US");
@@ -1817,14 +1768,12 @@ main(argc, argv)
    areastruct.gridon = True;
    areastruct.center = True;
    areastruct.bboxon = False;
-   areastruct.filter = ANY;
+   areastruct.filter = SEL_ANY;
    areastruct.selects = 0;
    areastruct.selectlist = NULL;
    areastruct.manhatn = False;
    areastruct.boxedit = MANHATTAN;
    areastruct.lastbackground = NULL;
-   areastruct.hlevels = 0;
-   areastruct.hierarchy = (objinstptr *) malloc(sizeof(objinstptr));
    areastruct.editstack = (objectptr) malloc(sizeof(object));
    initmem(areastruct.editstack);
 
@@ -1840,7 +1789,6 @@ main(argc, argv)
    xobjs.delbuffer.library = (objectptr *) malloc(DELBUFSIZE * sizeof(objectptr));
    xobjs.pages = PAGES;
 
-   shiftstate = False;
    fontcount = 0;
    fonts = (fontinfo *) malloc(sizeof(fontinfo));
 
@@ -1850,16 +1798,16 @@ main(argc, argv)
    areastruct.topobject->position.x = 0;
    areastruct.topobject->position.y = 0;
    areastruct.topobject->params = NULL;
-   areastruct.topobject->num_params = (uchar)0;
-   areastruct.MatStackDepth = 1; /* Top transformation matrix */
-   areastruct.MatStack = (Matrix *)malloc(sizeof(Matrix)); /* original allocation */
-   *areastruct.hierarchy = areastruct.topobject;
+   areastruct.MatStack = NULL;
+   areastruct.hierarchy = malloc(sizeof(objectpair));
+   areastruct.hierarchy->thisinst = areastruct.topobject;
+   areastruct.hierarchy->nextpair = NULL;
 
    /* Initialization of objects requires values for the window width and height, */
    /* so set up the widgets and realize them first.				 */
 
    popups = 0;        /* no popup windows yet */
-   pushes = 0;        /* at the top of the hierarchy */
+   pushlist = NULL;   /* at the top of the hierarchy */
    beeper = 1;        /* Ring bell on certain warnings or errors */
 
    version = PROG_VERSION;
@@ -2106,9 +2054,6 @@ main(argc, argv)
    XtAddCallback(areastruct.area, XtNrelease, (XtCallbackProc)releasebutton, NULL);
    XtAddCallback(areastruct.area, XtNkeyDown, (XtCallbackProc)keyhandler, NULL);
 
-   XtAddEventHandler(areastruct.area, KeyReleaseMask, False,
-	(XtEventHandler)keyrelease, NULL);
-
    /* Setup callback routines for the scrollbar widgets */
 
    XtAddEventHandler(areastruct.scrollbarh, ButtonMotionMask, False,
@@ -2141,15 +2086,13 @@ main(argc, argv)
    XtRealizeWidget(top);
 
    {
-      char newprop[36];
-      Atom *protret; 
-      int countret, i;
-      char *aret;
+/*
       Atom type_ret;
       int fmt_ret;
-      unsigned long nitems_ret;
-      unsigned long bytes_after_ret;
+      u_long nitems_ret;
+      u_long bytes_after_ret;
       GNUstepWMAttributes attr;
+*/
 
 #define GSFullKeyboardEventsFlag (1<<1)
 
@@ -2167,7 +2110,7 @@ main(argc, argv)
          attr.reserved2=0; 
          attr.extra_flags &= !GSFullKeyboardEventsFlag;
          XChangeProperty(dpy, XtWindow(top), wmprop[1], wmprop[1],
-                    32, PropModeReplace,  (unsigned char *)(&attr),
+                    32, PropModeReplace,  (u_char *)(&attr),
                     sizeof(GNUstepWMAttributes)/sizeof(CARD32));
       }
       else
@@ -2297,6 +2240,10 @@ main(argc, argv)
    /* Load the (default) built-in set of objects 	       */
    /*----------------------------------------------------------*/
 
+#ifdef HAVE_PYTHON
+   init_interpreter();
+#endif
+
    loadrcfile();
 
    /*-----------------------------------------------------*/
@@ -2322,7 +2269,7 @@ main(argc, argv)
    /*---------------------------------------------------*/
 
    xobjs.save_interval = appdata.timeout;
-   xobjs.timeout_id = XtAddTimeOut((unsigned long)60000 * xobjs.save_interval,
+   xobjs.timeout_id = XtAddTimeOut((u_long)60000 * xobjs.save_interval,
 	(XtTimerCallbackProc)savetemp, NULL);
 
    /*----------------------*/
