@@ -1,6 +1,6 @@
 /*----------------------------------------------------------------------*/
 /* xcircuit.h 								*/
-/* Copyright (c) 2000  Tim Edwards, Johns Hopkins University        	*/
+/* Copyright (c) 2001  Tim Edwards, Johns Hopkins University        	*/
 /*----------------------------------------------------------------------*/
 
 /*----------------------------------------------------------------------*/
@@ -16,7 +16,7 @@
 #define sign(a)			((a) <= 0 ? -1 : 1)
 #define max(a,b)		((a) < (b) ? (b) : (a))
 #define min(a,b)		((a) < (b) ? (a) : (b))
-#define Number(a)		(void *)((unsigned int) a)
+#define Number(a)		(void *)((u_int) a)
 #define ROT_UNIT (360 / RSTEPS) /* degrees per xcircuit rotational unit */
 
 /*----------------------------------------------------------------------*/
@@ -87,13 +87,22 @@
 
 /* selection from top-level object */
 
-#define objectselect(a)		selectobject(a, objectdata)
-#define easydraw(a, b)		geneasydraw(a, b, objectdata)
-#define drawselected(a)		gendrawselected(a, objectdata)
+#define objectselect(a)		selectobject(a, objectdata, NORMINST)
+#define easydraw(a, b)		geneasydraw(a, b, objectdata, NORMINST)
+#define drawselected(a)		gendrawselected(a, objectdata, NORMINST)
+
+/* normal choice of calling object instance */
+
+#define NORMINST ((pushlist == NULL) ? NULL : pushlist->thisinst)
 		   
 /*----------------------------------------------------------------------*/
 
-typedef unsigned char		uchar;
+#if !defined(_SYS_TYPES_H) || !defined(__USE_BSD)
+typedef unsigned char	u_char;
+typedef unsigned short	u_short;
+typedef unsigned int	u_int;
+typedef unsigned long	u_long;
+#endif
 
 typedef struct {
    float x, y;
@@ -185,35 +194,31 @@ UIPARAM_MODE
 #define FLIPINV		16
 
 /*----------------------------------------------------------------------*/
-/* Text script styles						   	*/
+/* Text string part: types						*/
 /*----------------------------------------------------------------------*/
 
-#define TEXT_ESC	27  /* escape character for defining text controls */
-#define CONTROL_START	32  /* first character used by controls */
-#define SUBSCRIPT	32
-#define SUPERSCRIPT	33
-#define NORMALSCRIPT	34
-#define UNDERLINE	35
-#define OVERLINE	36
-#define NOLINE		37
-#define BACKSPACE	38
-#define HALFSPACE	39
-#define QTRSPACE	40
-#define SPECIAL		41
-#define FONT_SCALE	42    /* inline font scale designator (begin) */
-#define END_SCALE	43    /* inline font scale designator (end) */
-#define PARAM_START	44
-#define PARAM_END	45
-#define RETURN		46    /* carriage-return character */
-#define FONT_COLOR	47
-#define END_COLOR	48
-#define KERN		49
-#define END_KERN	50
-
-#define FONT_START     100    /* character of first inline font designator */
-
-#define NO_OP	       255    /* padding for parameterized substrings */
+#define TEXT_STRING	 0  /* data is a text string 			*/
+#define SUBSCRIPT	 1  /* start subscript; no data			*/
+#define SUPERSCRIPT	 2  /* start superscript; no data		*/
+#define NORMALSCRIPT	 3  /* stop super-/subscript; no data		*/
+#define UNDERLINE	 4  /* start underline; no data			*/
+#define OVERLINE	 5  /* start overline; no data			*/
+#define NOLINE		 6  /* stop over-/underline; no data		*/
+#define TABSTOP	 	 7  /* insert tab stop position			*/
+#define TABFORWARD 	 8  /* insert tab stop position			*/
+#define TABBACKWARD 	 9  /* insert tab stop position			*/
+#define HALFSPACE	10  /* insert half-space; no data		*/
+#define QTRSPACE	11  /* insert quarter space; no data		*/
+#define RETURN		12  /* carriage-return character; no data	*/
+#define FONT_NAME       13  /* inline font designator; data = font name */
+#define FONT_SCALE	14  /* font scale change; data = scale		*/
+#define FONT_COLOR	15  /* font color change; data = color		*/
+#define KERN		16  /* set new kern values; data = kern x, y	*/
+#define PARAM_START	17  /* bounds a parameter; data = param #	*/
+#define PARAM_END	18  /* bounds a parameter; data = param #	*/
 
+#define SPECIAL		63  /* used only when called from menu		*/
+#define NULL_TYPE	255 /* used as a placeholder			*/
 
 /*----------------------------------------------------------------------*/
 /* Reset modes								*/
@@ -350,50 +355,102 @@ typedef XfPoint* fpointlist;
 #define ARC	8
 #define SPLINE  16
 #define PATH	32
-#define ANY	63  /* for selection mechanism only */
+#define SEL_ANY	63  /* for selection mechanism only */
+
+/*----------------------------------------------------------------------*/
+/* Labels are constructed of strings and executables 			*/
+/*----------------------------------------------------------------------*/
+
+typedef struct _stringpart *stringptr;
+
+typedef struct _stringpart {
+   stringptr	nextpart;
+   u_char	type;
+   union {
+     u_char	*string;
+     int	color;
+     int	font;
+     int	paramno;
+     float	scale;
+     short	kern[2];
+   }		data;
+} stringpart;
 
 /*----------------------------------------------------------------------*/
 /* structures of all main elements which can be displayed & manipulated	*/
 /*----------------------------------------------------------------------*/
 
-typedef struct _object *objectptr;
+/*----------------------------------------------------------------------*/
+/* Generic element type	is a superset of all elements.			*/	
+/*----------------------------------------------------------------------*/
 
 typedef struct {
-   uchar	type;		/* type is LABEL, POLYGON, etc., from below */
+   u_char	type;		/* type is LABEL, POLYGON, etc., from below */
    int		color;
 } generic, *genericptr;		/* (convenience function for retypecasting) */
 
+/*----------------------------------------------------------------------*/
+/* Object & object instance parameter structure				*/
+/* (Add types as necessary)						*/
+/*----------------------------------------------------------------------*/
+
+enum paramtypes {XC_SHORT = 0, XC_STRING};
+
 typedef struct {
-   uchar	type;
+   u_char	type;		/* type is from paramtypes list above */
+   union {
+      stringpart *string;
+      short	 svalue;
+   } parameter;			/* default or substitution value */
+} oparam, *oparamptr;
+
+/*----------------------------------------------------------------------*/
+/* Object instance type							*/
+/*----------------------------------------------------------------------*/
+
+typedef struct _xcobject *objectptr;
+
+typedef struct {
+   u_char	type;
    int		color;
    XPoint	position;
    short	rotation;
    float	scale;
-   void		**params;	/* parameter types determined from thisobject */
-   uchar	num_params;	/* number of parameters currently allocated */
+   oparamptr    *params;	/* parameter substitutions for this instance */
    objectptr	thisobject;
 } objinst, *objinstptr;
 
-typedef struct {
+typedef struct _objectpair *objectpairptr;
+
+typedef struct _objectpair {
    objectptr    thisobject;
    objinstptr	thisinst;
+   objectpairptr nextpair;
 } objectpair;
 
+/*----------------------------------------------------------------------*/
+/* Label								*/
+/*----------------------------------------------------------------------*/
+
 typedef struct {
-   uchar	type;
+   u_char	type;
    int		color;
    XPoint	position;
    short	rotation;
    float	scale;
    short	justify;
 #ifdef SCHEMA
-   uchar	pin;
+   u_char	pin;
 #endif
-   uchar	*string;
+   stringpart	*string;
 } label, *labelptr;
 
+/*----------------------------------------------------------------------*/
+/* Polygon								*/
+/*----------------------------------------------------------------------*/
+
 typedef struct {
-   uchar	type;
+   u_char	type;
    int		color;
    short	style;
    float	width;
@@ -401,8 +458,12 @@ typedef struct {
    pointlist	points;
 } polygon, *polyptr;
 
+/*----------------------------------------------------------------------*/
+/* Bezier Curve								*/
+/*----------------------------------------------------------------------*/
+
 typedef struct {
-   uchar	type;
+   u_char	type;
    int		color;
    short	style;
    float	width;
@@ -411,8 +472,12 @@ typedef struct {
    XfPoint	points[INTSEGS];
 } spline, *splineptr;
 
+/*----------------------------------------------------------------------*/
+/* Arc									*/
+/*----------------------------------------------------------------------*/
+
 typedef struct {
-   uchar	type;
+   u_char	type;
    int		color;
    short	style;
    float	width;
@@ -426,8 +491,12 @@ typedef struct {
    XfPoint	points[RSTEPS + 1];
 } arc, *arcptr;
 
+/*----------------------------------------------------------------------*/
+/* Path									*/
+/*----------------------------------------------------------------------*/
+
 typedef struct {
-   uchar	type;
+   u_char	type;
    int		color;
    short	style;
    float	width;
@@ -435,44 +504,61 @@ typedef struct {
    genericptr	*plist;		/* to be retypecast to polygon, arc, or spline */
 } path, *pathptr;
 
-/*-----------------------------------------------------------------------*/
-/* Netlist structures for schematic capture				 */
-/* (courtesy of Chow Seong Hwai, 3/20/98)				 */
-/*-----------------------------------------------------------------------*/
+/*----------------------------------------------------------------------*/
+/* Netlist structures for schematic capture				*/
+/*----------------------------------------------------------------------*/
 
 #ifdef SCHEMA
 
+/* Linked polygon list */
+
+typedef struct _Polylist *PolylistPtr;
+typedef struct _Polylist
+{
+   polyptr poly;
+   PolylistPtr next;
+} Polylist;
+
+/* Linked label list */
+
+typedef struct _Labellist *LabellistPtr;
+typedef struct _Labellist
+{
+   labelptr label;
+   LabellistPtr next;
+} Labellist;
+
+/* List of object's networks */
+
 typedef struct _Netlist *NetlistPtr;
 typedef struct _Netlist
 {
-   polyptr poly;
    int netid;
+   PolylistPtr polygons;
+   LabellistPtr labels;
+   stringpart *localpin;
    NetlistPtr next;
 } Netlist;
 
-typedef struct _Pinlist *PinlistPtr;
-typedef struct _Pinlist
-{
-   labelptr pin;
-   char *pinx;
-   int netid;
-   PinlistPtr next;
-} Pinlist;
+/* List of object's I/O ports */
 
-typedef struct _Paramlist *ParamlistPtr;
-typedef struct _Paramlist
+typedef struct _Portlist *PortlistPtr;
+typedef struct _Portlist
 {
+   int portid;
    int netid;
-   labelptr pin;
-   ParamlistPtr next;
-} Paramlist;
+   PortlistPtr next;
+} Portlist;
+
+/* List of calls to instances of objects */
+/* or subcircuit objects.		 */
 
 typedef struct _Calllist *CalllistPtr;
 typedef struct _Calllist
 {
    objectptr callobj;
-   objinstptr callno;
-   ParamlistPtr params;
+   objinstptr callinst;
+   PortlistPtr ports;
    CalllistPtr next;
 } Calllist;
 
@@ -485,7 +571,7 @@ struct Pnet {
 };
 
 struct Pstr {
-   char *string;
+   stringpart *string;
    struct Pstr *next;
 };
 
@@ -497,24 +583,11 @@ struct Ptab {
 
 #endif
 
-/*----------------------------------------------------------------------*/
-/* Object parameter structure						*/
-/*----------------------------------------------------------------------*/
-
-enum paramtypes {XC_SHORT = 0, XC_STRING};
-
-typedef struct _oparam {
-   uchar	type;		/* type is from paramtypes list above */
-   short	*pdefault;	/* default value */
-   uchar	number;		/* number of locations to replace */
-   uchar	**position;	/* list of locations to replace (fragile!) */
-} oparam, *oparamptr;
-
 /*----------------------------------------------------------------------*/
 /* Main object structure						*/
 /*----------------------------------------------------------------------*/
 
-typedef struct _object {
+typedef struct _xcobject {
    char		name[80];
    Boolean	hidden;
    float	viewscale;
@@ -523,18 +596,18 @@ typedef struct _object {
    Dimension	width, height;  /* information  */
    short	parts;
    genericptr	*plist;	 	/* to be retypecast to label, polygon, etc. */
-   uchar	num_params;	/* number of calling parameters */
-   oparamptr	*params;	/* list of parameter values */
+   u_char	num_params;	/* number of calling parameters */
+   oparamptr	*params;	/* list of parameter defaults */
    
 #ifdef SCHEMA
    XPoint	lleft2;		 /* bounding box info excluding pins */
    Dimension	width2, height2;
 
-   uchar	schemtype;
+   u_char	schemtype;
    objectptr	symschem;	 /* schematic page support */
 
    NetlistPtr   netlist;
-   PinlistPtr   pinlist;
+   PortlistPtr  portlist;
    CalllistPtr  calllist;
    short traversed;
 #endif
@@ -544,17 +617,18 @@ typedef struct _object {
 /* Transformation matrices						*/
 /*----------------------------------------------------------------------*/
 
-typedef struct {
+typedef struct _matrix *Matrixptr;
+
+typedef struct _matrix {
    float a, b, c, d, e, f;
+   Matrixptr nextmatrix;
 } Matrix;
 
 /*----------------------------------------------------------------------*/
 /* Some convenience functions for matrix manipulation			*/
 /*----------------------------------------------------------------------*/
 
-#define DCTM  &areastruct.MatStack[areastruct.MatStackDepth - 1]
-#define DCTM2 &areastruct.MatStack[areastruct.MatStackDepth - 2]
-#define UPopCTM() areastruct.MatStackDepth--
+#define DCTM  areastruct.MatStack
 
 /*----------------------------------------------------------------------*/
 /* button tap/press definitions 					*/
@@ -657,7 +731,7 @@ typedef struct {
    char *psname;
    char *family;
    float scale;
-   unsigned short flags;
+   u_short flags;
    objectptr *encoding;
 } fontinfo;
 
@@ -669,6 +743,50 @@ typedef struct {
    XPoint upperright;
 } psbkground;
 
+/*----------------------------------------------------------------------*/
+/* Key macro information						*/
+/*----------------------------------------------------------------------*/
+
+typedef struct _keybinding *keybindingptr;
+
+typedef struct _keybinding {
+   int keywstate;
+   int function;
+   short value;
+   keybindingptr nextbinding;
+} keybinding;
+
+/*----------------------------------------------------------------------*/
+/* Enumeration of functions available for binding to keys/buttons	*/
+/*----------------------------------------------------------------------*/
+
+enum {
+   XCF_ENDDATA = -2, XCF_SPACER,
+   XCF_Page, XCF_Justify, XCF_Superscript, XCF_Subscript, XCF_Normalscript,
+   XCF_Nextfont, XCF_Boldfont, XCF_Italicfont, XCF_Normalfont, XCF_Underline,
+   XCF_Overline, XCF_ISO_Encoding, XCF_Return, XCF_Halfspace, XCF_Quarterspace,
+   XCF_Special, XCF_TabStop, XCF_TabForward, XCF_TabBackward,
+   XCF_Text_Return, XCF_Text_Delete, XCF_Text_Right, XCF_Text_Left,
+   XCF_Text_Home, XCF_Text_End, XCF_Return_Char,
+   XCF_Parameter, XCF_Edit_Break, XCF_Edit_Delete, XCF_Edit_Insert,
+   XCF_Edit_Next, XCF_Attach, XCF_Next_Library, XCF_Library_Directory,
+   XCF_Library_Move, XCF_Library_Copy, XCF_Library_Edit, XCF_Library_Delete,
+   XCF_Library_Duplicate, XCF_Library_Hide, XCF_Page_Directory, XCF_Library_Pop,
+   XCF_Help, XCF_Redraw, XCF_View, XCF_Zoom_In, XCF_Zoom_Out, XCF_Pan,
+   XCF_Double_Snap, XCF_Halve_Snap, XCF_Pan_Left, XCF_Pan_Right, XCF_Pan_Up,
+   XCF_Pan_Down, XCF_Write, XCF_Rotate, XCF_Flip_X, XCF_Flip_Y, XCF_Snap,
+   XCF_Pop, XCF_Push, XCF_Delete, XCF_Select, XCF_Box, XCF_Arc, XCF_Text,
+   XCF_Exchange, XCF_Copy, XCF_Join, XCF_Unjoin, XCF_Spline, XCF_Edit,
+   XCF_Undelete, XCF_Select_Save, XCF_Unselect, XCF_Dashed, XCF_Dotted,
+   XCF_Solid, XCF_Prompt, XCF_Dot, XCF_Wire, XCF_Nothing, XCF_Exit,
+
+#ifdef SCHEMA
+   XCF_Netlist, XCF_Swap, XCF_Pin_Label, XCF_Info_Label, XCF_Connectivity,
+   XCF_Sim, XCF_SPICE, XCF_PCB,
+#endif
+   NUM_FUNCTIONS
+};
+
 /*----------------------------------------------------------------------*/
 /* Per-drawing-page parameters						*/
 /*----------------------------------------------------------------------*/
@@ -738,11 +856,9 @@ typedef struct {
    short	selects;
    short	*selectlist;
    objinstptr	topobject;
-   objinstptr	*hierarchy;
-   short	hlevels;
+   objectpair	*hierarchy;
    objectptr	editstack;
-   Matrix	*MatStack;
-   short	MatStackDepth;
+   Matrixptr	MatStack;
    short	editcycle;
    short	editpart, editsubpart;
    char		*lastbackground;
@@ -771,3 +887,32 @@ typedef struct {
 } Globaldata;
 
 /*----------------------------------------------------------------------*/
+/* structures previously defined in menudefs.h				*/
+/*----------------------------------------------------------------------*/
+/*----------------------------------------------------------------------*/
+/* Menu Definitions for hierarchical pulldown menus                     */
+/*----------------------------------------------------------------------*/
+
+typedef struct _menustruct *menuptr;
+
+typedef struct _menustruct {
+        char *name;
+        menuptr submenu;
+        short size;
+        XtCallbackProc func;
+        void *passeddata;
+} menustruct;
+
+/*----------------------------------------------------------------------*/
+/* Structure for calling routines from the Toolbar icons		*/
+/*----------------------------------------------------------------------*/
+
+typedef struct _toolbarstruct {
+	char *name;
+	char **icon_data;
+        XtCallbackProc func;
+        void *passeddata;
+	char *hint;
+} toolbarstruct;
+
+/*----------------------------------------------------------------------*/
